@@ -1,3 +1,25 @@
+/*****************************************************************************
+ *                                  _   _ ____  _     
+ *  Project                     ___| | | |  _ \| |    
+ *                             / __| | | | |_) | |    
+ *                            | (__| |_| |  _ <| |___ 
+ *                             \___|\___/|_| \_\_____|
+ *
+ * Copyright (C) 2001, Daniel Stenberg, <daniel@haxx.se>, et al.
+ *
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
+ *
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
+ *
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
+ *
+ * $Id: cookie.c,v 1.20 2001/09/10 07:43:08 bagder Exp $
+ *****************************************************************************/
 
 /***
 
@@ -64,6 +86,7 @@ Example set of cookies:
 #include "cookie.h"
 #include "getdate.h"
 #include "strequal.h"
+#include "strtok.h"
 
 /* The last #include file should be: */
 #ifdef MALLOCDEBUG
@@ -72,15 +95,16 @@ Example set of cookies:
 
 /****************************************************************************
  *
- * cookie_add()
+ * Curl_cookie_add()
  *
  * Add a single cookie line to the cookie keeping object.
  *
  ***************************************************************************/
 
-struct Cookie *cookie_add(struct CookieInfo *c,
-                          bool httpheader, /* TRUE if HTTP header-style line */
-                          char *lineptr) /* first non-space of the line */
+struct Cookie *
+Curl_cookie_add(struct CookieInfo *c,
+                bool httpheader, /* TRUE if HTTP header-style line */
+                char *lineptr) /* first non-space of the line */
 {
   struct Cookie *clist;
   char what[MAX_COOKIE_LINE];
@@ -88,6 +112,7 @@ struct Cookie *cookie_add(struct CookieInfo *c,
   char *ptr;
   char *semiptr;
   struct Cookie *co;
+  struct Cookie *lastc=NULL;
   time_t now = time(NULL);
   bool replace_old = FALSE;
 
@@ -105,13 +130,11 @@ struct Cookie *cookie_add(struct CookieInfo *c,
     semiptr=strchr(lineptr, ';'); /* first, find a semicolon */
     ptr = lineptr;
     do {
-      if(semiptr)
-        *semiptr='\0'; /* zero terminate for a while */
       /* we have a <what>=<this> pair or a 'secure' word here */
       if(strchr(ptr, '=')) {
         name[0]=what[0]=0; /* init the buffers */
         if(1 <= sscanf(ptr, "%" MAX_NAME_TXT "[^=]=%"
-                       MAX_COOKIE_LINE_TXT "[^\r\n]",
+                       MAX_COOKIE_LINE_TXT "[^;\r\n]",
                        name, what)) {
           /* this is a legal <what>=<this> pair */
           if(strequal("path", name)) {
@@ -145,27 +168,27 @@ struct Cookie *cookie_add(struct CookieInfo *c,
             co->name = strdup(name);
             co->value = strdup(what);
           }
-          else
-            ;/* this is the second (or more) name we don't know
-                about! */
+          /*
+            else this is the second (or more) name we don't know
+            about! */
         }
         else {
           /* this is an "illegal" <what>=<this> pair */
         }
       }
       else {
-        if(sscanf(ptr, "%" MAX_COOKIE_LINE_TXT "[^\r\n]",
+        if(sscanf(ptr, "%" MAX_COOKIE_LINE_TXT "[^;\r\n]",
                   what)) {
           if(strequal("secure", what))
             co->secure = TRUE;
-          else
-            ; /* unsupported keyword without assign! */
+          /* else,
+             unsupported keyword without assign! */
+
         }
       }
       if(!semiptr)
         continue; /* we already know there are no more cookies */
 
-      *semiptr=';'; /* put the semicolon back */
       ptr=semiptr+1;
       while(ptr && *ptr && isspace((int)*ptr))
         ptr++;
@@ -176,6 +199,7 @@ struct Cookie *cookie_add(struct CookieInfo *c,
     /* This line is NOT a HTTP header style line, we do offer support for
        reading the odd netscape cookies-file format here */
     char *firstptr;
+    char *tok_buf;
     int fields;
 
     if(lineptr[0]=='#') {
@@ -191,7 +215,7 @@ struct Cookie *cookie_add(struct CookieInfo *c,
     if(ptr)
       *ptr=0; /* clear it */
 
-    firstptr=strtok(lineptr, "\t"); /* first tokenize it on the TAB */
+    firstptr=strtok_r(lineptr, "\t", &tok_buf); /* first tokenize it on the TAB */
 
     /* Here's a quick check to eliminate normal HTTP-headers from this */
     if(!firstptr || strchr(firstptr, ':')) {
@@ -201,17 +225,39 @@ struct Cookie *cookie_add(struct CookieInfo *c,
 
     /* Now loop through the fields and init the struct we already have
        allocated */
-    for(ptr=firstptr, fields=0; ptr; ptr=strtok(NULL, "\t"), fields++) {
+    for(ptr=firstptr, fields=0; ptr; ptr=strtok_r(NULL, "\t", &tok_buf), fields++) {
       switch(fields) {
       case 0:
         co->domain = strdup(ptr);
         break;
       case 1:
-        /* what _is_ this field for? */
+        /* This field got its explanation on the 23rd of May 2001 by
+           Andrs Garca:
+
+           flag: A TRUE/FALSE value indicating if all machines within a given
+           domain can access the variable. This value is set automatically by
+           the browser, depending on the value you set for the domain.
+
+           As far as I can see, it is set to true when the cookie says
+           .domain.com and to false when the domain is complete www.domain.com
+
+           We don't currently take advantage of this knowledge.
+        */
+        co->field1=strequal(ptr, "TRUE")+1; /* store information */
         break;
       case 2:
-        co->path = strdup(ptr);
-        break;
+        /* It turns out, that sometimes the file format allows the path
+           field to remain not filled in, we try to detect this and work
+           around it! Andrs Garca made us aware of this... */
+        if (strcmp("TRUE", ptr) && strcmp("FALSE", ptr)) {
+          /* only if the path doesn't look like a boolean option! */
+          co->path = strdup(ptr);
+          break;
+        }
+        /* this doesn't look like a path, make one up! */
+        co->path = strdup("/");
+        fields++; /* add a field and fall down to secure */
+        /* FALLTHROUGH */
       case 3:
         co->secure = strequal(ptr, "TRUE");
         break;
@@ -246,6 +292,8 @@ struct Cookie *cookie_add(struct CookieInfo *c,
 
   }
 
+  co->livecookie = c->running;
+
   /* now, we have parsed the incoming line, we must now check if this
      superceeds an already existing cookie, which it may if the previous have
      the same domain and path as this */
@@ -280,6 +328,26 @@ struct Cookie *cookie_add(struct CookieInfo *c,
         
       }
 
+      if(replace_old && !co->livecookie && clist->livecookie) {
+        /* Both cookies matched fine, except that the already present
+           cookie is "live", which means it was set from a header, while
+           the new one isn't "live" and thus only read from a file. We let
+           live cookies stay alive */
+
+        /* Free the newcomer and get out of here! */
+        if(co->domain)
+          free(co->domain);
+        if(co->path)
+          free(co->path);
+        if(co->name)
+          free(co->name);
+        if(co->value)
+          free(co->value);
+
+        free(co);
+        return NULL;
+      }
+
       if(replace_old) {
         co->next = clist->next; /* get the next-pointer first */
 
@@ -304,39 +372,51 @@ struct Cookie *cookie_add(struct CookieInfo *c,
       }
 
     }
+    lastc = clist;
     clist = clist->next;
   }
 
   if(!replace_old) {
-
-    /* first, point to our "next" */
-    co->next = c->cookies;
-    /* then make ourselves first in the list */
-    c->cookies = co;
+    /* then make the last item point on this new one */
+    if(lastc)
+      lastc->next = co;
+    else
+      c->cookies = co;
   }
+
+  c->numcookies++; /* one more cookie in the jar */
+
   return co;
 }
 
 /*****************************************************************************
  *
- * cookie_init()
+ * Curl_cookie_init()
  *
  * Inits a cookie struct to read data from a local file. This is always
  * called before any cookies are set. File may be NULL.
  *
  ****************************************************************************/
-struct CookieInfo *cookie_init(char *file)
+struct CookieInfo *Curl_cookie_init(char *file, struct CookieInfo *inc)
 {
   char line[MAX_COOKIE_LINE];
   struct CookieInfo *c;
   FILE *fp;
   bool fromfile=TRUE;
   
-  c = (struct CookieInfo *)malloc(sizeof(struct CookieInfo));
-  if(!c)
-    return NULL; /* failed to get memory */
-  memset(c, 0, sizeof(struct CookieInfo));
-  c->filename = strdup(file?file:"none"); /* copy the name just in case */
+  if(NULL == inc) {
+    /* we didn't get a struct, create one */
+    c = (struct CookieInfo *)malloc(sizeof(struct CookieInfo));
+    if(!c)
+      return NULL; /* failed to get memory */
+    memset(c, 0, sizeof(struct CookieInfo));
+    c->filename = strdup(file?file:"none"); /* copy the name just in case */
+  }
+  else {
+    /* we got an already existing one, use that */
+    c = inc;
+  }
+  c->running = FALSE; /* this is not running, this is init */
 
   if(strequal(file, "-")) {
     fp = stdin;
@@ -346,34 +426,35 @@ struct CookieInfo *cookie_init(char *file)
     fp = file?fopen(file, "r"):NULL;
 
   if(fp) {
+    char *lineptr;
+    bool headerline;
     while(fgets(line, MAX_COOKIE_LINE, fp)) {
       if(strnequal("Set-Cookie:", line, 11)) {
         /* This is a cookie line, get it! */
-        char *lineptr=&line[11];
-        while(*lineptr && isspace((int)*lineptr))
-          lineptr++;
-
-        cookie_add(c, TRUE, lineptr);
+        lineptr=&line[11];
+        headerline=TRUE;
       }
       else {
-        /* This might be a netscape cookie-file line, get it! */
-        char *lineptr=line;
-        while(*lineptr && isspace((int)*lineptr))
-          lineptr++;
-
-        cookie_add(c, FALSE, lineptr);
+        lineptr=line;
+        headerline=FALSE;
       }
+      while(*lineptr && isspace((int)*lineptr))
+        lineptr++;
+
+      Curl_cookie_add(c, headerline, lineptr);
     }
     if(fromfile)
       fclose(fp);
   }
 
+  c->running = TRUE; /* now, we're running */
+
   return c;
 }
 
 /*****************************************************************************
  *
- * cookie_getlist()
+ * Curl_cookie_getlist()
  *
  * For a given host and path, return a linked list of cookies that the
  * client should send to the server if used now. The secure boolean informs
@@ -383,8 +464,8 @@ struct CookieInfo *cookie_init(char *file)
  *
  ****************************************************************************/
 
-struct Cookie *cookie_getlist(struct CookieInfo *c,
-			      char *host, char *path, bool secure)
+struct Cookie *Curl_cookie_getlist(struct CookieInfo *c,
+                                   char *host, char *path, bool secure)
 {
    struct Cookie *newco;
    struct Cookie *co;
@@ -445,13 +526,13 @@ struct Cookie *cookie_getlist(struct CookieInfo *c,
 
 /*****************************************************************************
  *
- * cookie_freelist()
+ * Curl_cookie_freelist()
  *
- * Free a list previously returned by cookie_getlist();
+ * Free a list of cookies previously returned by Curl_cookie_getlist();
  *
  ****************************************************************************/
 
-void cookie_freelist(struct Cookie *co)
+void Curl_cookie_freelist(struct Cookie *co)
 {
    struct Cookie *next;
    if(co) {
@@ -466,12 +547,12 @@ void cookie_freelist(struct Cookie *co)
 
 /*****************************************************************************
  *
- * cookie_cleanup()
+ * Curl_cookie_cleanup()
  *
  * Free a "cookie object" previous created with cookie_init().
  *
  ****************************************************************************/
-void cookie_cleanup(struct CookieInfo *c)
+void Curl_cookie_cleanup(struct CookieInfo *c)
 {
    struct Cookie *co;
    struct Cookie *next;
@@ -505,3 +586,101 @@ void cookie_cleanup(struct CookieInfo *c)
    }
 }
 
+/*
+ * Curl_cookie_output()
+ *
+ * Writes all internally known cookies to the specified file. Specify
+ * "-" as file name to write to stdout.
+ *
+ * The function returns non-zero on write failure.
+ */
+int Curl_cookie_output(struct CookieInfo *c, char *dumphere)
+{
+  struct Cookie *co;
+  FILE *out;
+  bool use_stdout=FALSE;
+
+  if((NULL == c) || (0 == c->numcookies))
+    /* If there are no known cookies, we don't write or even create any
+       destination file */
+    return 0;
+
+  if(strequal("-", dumphere)) {
+    /* use stdout */
+    out = stdout;
+    use_stdout=TRUE;
+  }
+  else {
+    out = fopen(dumphere, "w");
+    if(!out)
+      return 1; /* failure */
+  }
+
+  if(c) {
+    fputs("# Netscape HTTP Cookie File\n"
+          "# http://www.netscape.com/newsref/std/cookie_spec.html\n"
+          "# This is generated by libcurl! Edit on your own risk.\n\n",
+          out);
+    co = c->cookies;
+     
+    while(co) {
+      fprintf(out,
+              "%s\t" /* domain */
+              "%s\t" /* field1 */
+              "%s\t" /* path */
+              "%s\t" /* secure */
+              "%u\t" /* expires */
+              "%s\t" /* name */
+              "%s\n", /* value */
+              co->domain,
+              co->field1==2?"TRUE":"FALSE",
+              co->path,
+              co->secure?"TRUE":"FALSE",
+              (unsigned int)co->expires,
+              co->name,
+              co->value);
+
+      co=co->next;
+    }
+  }
+
+  if(!use_stdout)
+    fclose(out);
+
+  return 0;
+}
+
+#ifdef CURL_COOKIE_DEBUG
+
+/*
+ * On my Solaris box, this command line builds this test program:
+ *
+ * gcc -g -o cooktest -DCURL_COOKIE_DEBUG -DHAVE_CONFIG_H -I.. -I../include cookie.c strequal.o getdate.o memdebug.o mprintf.o strtok.o -lnsl -lsocket
+ *
+ */
+
+int main(int argc, char **argv)
+{
+  struct CookieInfo *c=NULL;
+  if(argc>1) {
+    c = Curl_cookie_init(argv[1], c);
+    Curl_cookie_add(c, TRUE, "PERSONALIZE=none;expires=Monday, 13-Jun-1988 03:04:55 GMT; domain=.fidelity.com; path=/ftgw; secure");
+    Curl_cookie_add(c, TRUE, "foobar=yes; domain=.haxx.se; path=/looser;");
+    c = Curl_cookie_init(argv[1], c);
+
+    Curl_cookie_output(c);
+    Curl_cookie_cleanup(c);
+    return 0;
+  }
+  return 1;
+}
+
+#endif
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
