@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: dict.c,v 1.14 2001/03/14 14:11:11 bagder Exp $
+ * $Id: dict.c,v 1.17 2001/09/07 04:01:32 bumblebury Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -80,11 +80,6 @@
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
 
-CURLcode Curl_dict_done(struct connectdata *conn)
-{
-  return CURLE_OK;
-}
-
 CURLcode Curl_dict(struct connectdata *conn)
 {
   int nth;
@@ -95,7 +90,7 @@ CURLcode Curl_dict(struct connectdata *conn)
   char *nthdef = NULL; /* This is not part of the protocol, but required
                           by RFC 2229 */
   CURLcode result=CURLE_OK;
-  struct UrlData *data=conn->data;
+  struct SessionHandle *data=conn->data;
 
   char *path = conn->path;
   long *bytecount = &conn->bytecount;
@@ -129,10 +124,10 @@ CURLcode Curl_dict(struct connectdata *conn)
       failf(data, "lookup word is missing\n");
     }
     if ((database == NULL) || (*database == (char)0)) {
-      database = "!";
+      database = (char *)"!";
     }
     if ((strategy == NULL) || (*strategy == (char)0)) {
-      strategy = ".";
+      strategy = (char *)".";
     }
     if ((nthdef == NULL) || (*nthdef == (char)0)) {
       nth = 0;
@@ -182,7 +177,7 @@ CURLcode Curl_dict(struct connectdata *conn)
       failf(data, "lookup word is missing\n");
     }
     if ((database == NULL) || (*database == (char)0)) {
-      database = "!";
+      database = (char *)"!";
     }
     if ((nthdef == NULL) || (*nthdef == (char)0)) {
       nth = 0;
@@ -237,3 +232,11 @@ CURLcode Curl_dict(struct connectdata *conn)
 
   return CURLE_OK;
 }
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
