@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: escape.c,v 1.13 2001/04/07 18:35:28 bagder Exp $
+ * $Id: escape.c,v 1.17 2001/09/07 04:01:32 bumblebury Exp $
  *****************************************************************************/
 
 /* Escape and unescape URL encoding in strings. The functions return a new
@@ -37,9 +37,9 @@
 #include "memdebug.h"
 #endif
 
-char *curl_escape(char *string, int length)
+char *curl_escape(const char *string, int length)
 {
-  int alloc = (length?length:strlen(string))+1;  
+  int alloc = (length?length:(int)strlen(string))+1;  
   char *ns = malloc(alloc);
   unsigned char in;
   int newlen = alloc;
@@ -54,28 +54,15 @@ char *curl_escape(char *string, int length)
             !(in >= 'A' && in <= 'Z') &&
             !(in >= '0' && in <= '9')) {
       /* encode it */
-      if(('%' == in) &&
-         (length>=2) &&
-         isxdigit((int)string[1]) &&
-         isxdigit((int)string[2]) ) {
-        /*
-         * This is an already encoded letter, leave it!
-         */
-        memcpy(&ns[index], string, 3);
-        string+=2;
+      newlen += 2; /* the size grows with two, since this'll become a %XX */
+      if(newlen > alloc) {
+        alloc *= 2;
+        ns = realloc(ns, alloc);
+        if(!ns)
+          return NULL;
       }
-      else {
-        /* encode this now */
+      sprintf(&ns[index], "%%%02X", in);
 
-        newlen += 2; /* the size grows with two, since this'll become a %XX */
-        if(newlen > alloc) {
-          alloc *= 2;
-          ns = realloc(ns, alloc);
-          if(!ns)
-            return NULL;
-        }
-        sprintf(&ns[index], "%%%02X", in);
-      }
       index+=3;
     }
     else {
@@ -88,9 +75,9 @@ char *curl_escape(char *string, int length)
   return ns;
 }
 
-char *curl_unescape(char *string, int length)
+char *curl_unescape(const char *string, int length)
 {
-  int alloc = (length?length:strlen(string))+1;
+  int alloc = (length?length:(int)strlen(string))+1;
   char *ns = malloc(alloc);
   unsigned char in;
   int index=0;
@@ -123,3 +110,11 @@ char *curl_unescape(char *string, int length)
   return ns;
   
 }
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
