@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: http_chunks.c,v 1.5 2001/03/13 23:29:53 bagder Exp $
+ * $Id: http_chunks.c,v 1.8 2001/09/07 04:01:32 bumblebury Exp $
  *****************************************************************************/
 #include "setup.h"
 
@@ -44,7 +44,7 @@
  * Chunk format (simplified):
  *
  * <HEX SIZE>[ chunk extension ] CRLF
- * <DATA>
+ * <DATA> CRLF
  *
  * Highlights from RFC2616 section 3.6 say:
 
@@ -96,8 +96,8 @@ void Curl_httpchunk_init(struct connectdata *conn)
  */
 CHUNKcode Curl_httpchunk_read(struct connectdata *conn,
                               char *datap,
-                              ssize_t length,
-                              ssize_t *wrote)
+                              size_t length,
+                              size_t *wrote)
 {
   CURLcode result;
   struct Curl_chunker *ch = &conn->proto.http->chunk;
@@ -220,3 +220,11 @@ CHUNKcode Curl_httpchunk_read(struct connectdata *conn,
   }
   return CHUNKE_OK;
 }
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
