@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: ldap.c,v 1.12 2001/02/20 17:35:52 bagder Exp $
+ * $Id: ldap.c,v 1.15 2001/09/07 04:01:32 bumblebury Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -101,7 +101,7 @@ static void DynaClose(void)
 #endif
 }
 
-static void * DynaGetFunction(char *name)
+static void * DynaGetFunction(const char *name)
 {
   void *func = NULL;
 
@@ -116,16 +116,12 @@ static void * DynaGetFunction(char *name)
 
 static int WriteProc(void *param, char *text, int len)
 {
-  struct UrlData *data = (struct UrlData *)param;
+  struct SessionHandle *data = (struct SessionHandle *)param;
+  len = 0; /* prevent compiler warning */
   Curl_client_write(data, CLIENTWRITE_BODY, text, 0);
   return 0;
 }
 
-CURLcode Curl_ldap_done(struct connectdata *conn)
-{
-  return CURLE_OK;
-}
-
 /***********************************************************************
  */
 CURLcode Curl_ldap(struct connectdata *conn)
@@ -146,9 +142,9 @@ CURLcode Curl_ldap(struct connectdata *conn)
   void *entryIterator;
 
   int ldaptext;
-  struct UrlData *data=conn->data;
+  struct SessionHandle *data=conn->data;
   
-  infof(data, "LDAP: %s %s\n", data->url);
+  infof(data, "LDAP: %s %s\n", data->change.url);
 
   DynaOpen();
   if (libldap == NULL) {
@@ -156,7 +152,7 @@ CURLcode Curl_ldap(struct connectdata *conn)
     return CURLE_LIBRARY_NOT_FOUND;
   }
 
-  ldaptext = data->bits.ftp_ascii; /* This is a dirty hack */
+  ldaptext = data->set.ftp_ascii; /* This is a dirty hack */
   
   /* The types are needed because ANSI C distinguishes between
    * pointer-to-object (data) and pointer-to-function.
@@ -177,12 +173,12 @@ CURLcode Curl_ldap(struct connectdata *conn)
 	  conn->hostname, conn->port);
     status = CURLE_COULDNT_CONNECT;
   } else {
-    rc = ldap_simple_bind_s(server, data->user, data->passwd);
+    rc = ldap_simple_bind_s(server, data->state.user, data->state.passwd);
     if (rc != 0) {
       failf(data, "LDAP: %s", ldap_err2string(rc));
       status = CURLE_LDAP_CANNOT_BIND;
     } else {
-      rc = ldap_url_search_s(server, data->url, 0, &result);
+      rc = ldap_url_search_s(server, data->change.url, 0, &result);
       if (rc != 0) {
 	failf(data, "LDAP: %s", ldap_err2string(rc));
 	status = CURLE_LDAP_SEARCH_FAILED;
@@ -194,7 +190,7 @@ CURLcode Curl_ldap(struct connectdata *conn)
 	    if (ldaptext) {
 	      rc = ldap_entry2text(server, NULL, entryIterator, NULL,
 				   NULL, NULL, WriteProc, data,
-				   "", 0, 0);
+				   (char *)"", 0, 0);
 	      if (rc != 0) {
 		failf(data, "LDAP: %s", ldap_err2string(rc));
 		status = CURLE_LDAP_SEARCH_FAILED;
@@ -202,7 +198,7 @@ CURLcode Curl_ldap(struct connectdata *conn)
 	    } else {
 	      rc = ldap_entry2html(server, NULL, entryIterator, NULL,
 				   NULL, NULL, WriteProc, data,
-				   "", 0, 0, NULL, NULL);
+				   (char *)"", 0, 0, NULL, NULL);
 	      if (rc != 0) {
 		failf(data, "LDAP: %s", ldap_err2string(rc));
 		status = CURLE_LDAP_SEARCH_FAILED;
@@ -217,3 +213,11 @@ CURLcode Curl_ldap(struct connectdata *conn)
   
   return status;
 }
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
