@@ -1,9 +1,18 @@
 /****************************************************************************
  *
- * $Id: mprintf.c,v 1.5 2000/11/21 13:18:30 bagder Exp $
+ * $Id: mprintf.c,v 1.12 2001/09/07 04:01:32 bumblebury Exp $
  *
  *************************************************************************
  *
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
+ * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
+ * MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE AUTHORS AND
+ * CONTRIBUTORS ACCEPT NO RESPONSIBILITY IN ANY CONCEIVABLE MANNER.
+ *
  * Purpose:
  *  A merge of Bjorn Reese's format() function and Daniel's dsprintf()
  *  1.0. A full blooded printf() clone with full support for <num>$
@@ -83,7 +92,7 @@
  *
  ****************************************************************************/
 
-static const char rcsid[] = "@(#)$Id: mprintf.c,v 1.5 2000/11/21 13:18:30 bagder Exp $";
+static const char rcsid[] = "@(#)$Id: mprintf.c,v 1.12 2001/09/07 04:01:32 bumblebury Exp $";
 
 /*
  * To test:
@@ -198,7 +207,7 @@ struct asprintf {
   size_t alloc; /* length of alloc */
 };
 
-int msprintf(char *buffer, const char *format, ...);
+int curl_msprintf(char *buffer, const char *format, ...);
 
 static int dprintf_DollarString(char *input, char **end)
 {
@@ -437,7 +446,7 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
 	case '1': case '2': case '3': case '4':
 	case '5': case '6': case '7': case '8': case '9':
 	  flags |= FLAGS_WIDTH;
-	  width = strtol(--fmt, &fmt, 10);
+	  width = strtol(fmt-1, &fmt, 10);
 	  break;
 	case '*':  /* Special case */
 	  flags |= FLAGS_WIDTHPARAM;
@@ -855,7 +864,7 @@ static int dprintf_formatf(
 	    p->flags &= (~FLAGS_ALT);
 	  }
 	  else {
-	    str = "";
+	    str = (char *)"";
 	    len = 0;
 	  }
 	}
@@ -946,11 +955,11 @@ static int dprintf_formatf(
 
 	if(width >= 0) {
 	  /* RECURSIVE USAGE */
-	  fptr += msprintf(fptr, "%d", width);
+	  fptr += curl_msprintf(fptr, "%d", width);
 	}
 	if(prec >= 0) {
 	  /* RECURSIVE USAGE */
-	  fptr += msprintf(fptr, ".%d", prec);
+	  fptr += curl_msprintf(fptr, ".%d", prec);
 	}
 	if (p->flags & FLAGS_LONG)
 	  strcat(fptr, "l");
@@ -1016,7 +1025,7 @@ static int addbyter(int output, FILE *data)
   return -1;
 }
 
-int msnprintf(char *buffer, size_t maxlength, const char *format, ...)
+int curl_msnprintf(char *buffer, size_t maxlength, const char *format, ...)
 {
   va_list ap_save; /* argument pointer */
   int retcode;
@@ -1036,7 +1045,7 @@ int msnprintf(char *buffer, size_t maxlength, const char *format, ...)
   return retcode;
 }
 
-int mvsnprintf(char *buffer, size_t maxlength, const char *format, va_list ap_save)
+int curl_mvsnprintf(char *buffer, size_t maxlength, const char *format, va_list ap_save)
 {
   int retcode;
   struct nsprintf info;
@@ -1083,7 +1092,7 @@ static int alloc_addbyter(int output, FILE *data)
 
 }
 
-char *maprintf(const char *format, ...)
+char *curl_maprintf(const char *format, ...)
 {
   va_list ap_save; /* argument pointer */
   int retcode;
@@ -1104,7 +1113,7 @@ char *maprintf(const char *format, ...)
     return NULL;
 }
 
-char *mvaprintf(const char *format, va_list ap_save)
+char *curl_mvaprintf(const char *format, va_list ap_save)
 {
   int retcode;
   struct asprintf info;
@@ -1131,7 +1140,7 @@ static int storebuffer(int output, FILE *data)
   return output; /* act like fputc() ! */
 }
 
-int msprintf(char *buffer, const char *format, ...)
+int curl_msprintf(char *buffer, const char *format, ...)
 {
   va_list ap_save; /* argument pointer */
   int retcode;
@@ -1142,9 +1151,11 @@ int msprintf(char *buffer, const char *format, ...)
   return retcode;
 }
 
+#ifndef WIN32 /* not needed on win32 */
 extern int fputc(int, FILE *);
+#endif
 
-int mprintf(const char *format, ...)
+int curl_mprintf(const char *format, ...)
 {
   int retcode;
   va_list ap_save; /* argument pointer */
@@ -1154,7 +1165,7 @@ int mprintf(const char *format, ...)
   return retcode;
 }
 
-int mfprintf(FILE *whereto, const char *format, ...)
+int curl_mfprintf(FILE *whereto, const char *format, ...)
 {
   int retcode;
   va_list ap_save; /* argument pointer */
@@ -1164,7 +1175,7 @@ int mfprintf(FILE *whereto, const char *format, ...)
   return retcode;
 }
 
-int mvsprintf(char *buffer, const char *format, va_list ap_save)
+int curl_mvsprintf(char *buffer, const char *format, va_list ap_save)
 {
   int retcode;
   retcode = dprintf_formatf(&buffer, storebuffer, format, ap_save);
@@ -1172,12 +1183,12 @@ int mvsprintf(char *buffer, const char *format, va_list ap_save)
   return retcode;
 }
 
-int mvprintf(const char *format, va_list ap_save)
+int curl_mvprintf(const char *format, va_list ap_save)
 {
   return dprintf_formatf(stdout, fputc, format, ap_save);
 }
 
-int mvfprintf(FILE *whereto, const char *format, va_list ap_save)
+int curl_mvfprintf(FILE *whereto, const char *format, va_list ap_save)
 {
   return dprintf_formatf(whereto, fputc, format, ap_save);
 }
@@ -1221,3 +1232,11 @@ int main()
 }
 
 #endif
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
