@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: netrc.c,v 1.12 2001/03/16 15:19:36 bagder Exp $
+ * $Id: netrc.c,v 1.16 2001/09/07 04:01:32 bumblebury Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -36,11 +36,14 @@
 #ifdef HAVE_PWD_H
 #include <pwd.h>
 #endif
-
+#ifdef VMS
+#include <unixlib.h>
+#endif
 
 #include <curl/curl.h>
 
 #include "strequal.h"
+#include "strtok.h"
 
 /* The last #include file should be: */
 #ifdef MALLOCDEBUG
@@ -87,8 +90,13 @@ int Curl_parsenetrc(char *host,
 #if defined(HAVE_GETPWUID) && defined(HAVE_GETEUID)
   struct passwd *pw;
   pw= getpwuid(geteuid());
-  if (pw)
+  if (pw) {
+#ifdef	VMS
+    home = decc$translate_vms(pw->pw_dir);
+#else
     home = pw->pw_dir;
+#endif
+  }
 #else
   void *pw=NULL;
 #endif
@@ -111,8 +119,9 @@ int Curl_parsenetrc(char *host,
   file = fopen(netrcbuffer, "r");
   if(file) {
     char *tok;
+	char *tok_buf;
     while(fgets(netrcbuffer, sizeof(netrcbuffer), file)) {
-      tok=strtok(netrcbuffer, " \t\n");
+      tok=strtok_r(netrcbuffer, " \t\n", &tok_buf);
       while(tok) {
 	switch(state) {
 	case NOTHING:
@@ -163,7 +172,7 @@ int Curl_parsenetrc(char *host,
 	  }
 	  break;
 	} /* switch (state) */
-	tok = strtok(NULL, " \t\n");
+	tok = strtok_r(NULL, " \t\n", &tok_buf);
       } /* while (tok) */
     } /* while fgets() */
 
@@ -192,3 +201,11 @@ int main(int argc, char **argv)
 }
 
 #endif
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
