@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sendf.c,v 1.23 2001/04/10 06:49:33 bagder Exp $
+ * $Id: sendf.c,v 1.29 2001/09/07 04:01:32 bumblebury Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -122,13 +122,13 @@ void curl_slist_free_all(struct curl_slist *list)
 
 /* Curl_infof() is for info message along the way */
 
-void Curl_infof(struct UrlData *data, char *fmt, ...)
+void Curl_infof(struct SessionHandle *data, const char *fmt, ...)
 {
   va_list ap;
-  if(data->bits.verbose) {
+  if(data->set.verbose) {
     va_start(ap, fmt);
-    fputs("* ", data->err);
-    vfprintf(data->err, fmt, ap);
+    fputs("* ", data->set.err);
+    vfprintf(data->set.err, fmt, ap);
     va_end(ap);
   }
 }
@@ -136,25 +136,20 @@ void Curl_infof(struct UrlData *data, char *fmt, ...)
 /* Curl_failf() is for messages stating why we failed, the LAST one will be
    returned for the user (if requested) */
 
-void Curl_failf(struct UrlData *data, char *fmt, ...)
+void Curl_failf(struct SessionHandle *data, const char *fmt, ...)
 {
   va_list ap;
   va_start(ap, fmt);
-  if(data->errorbuffer)
-    vsnprintf(data->errorbuffer, CURL_ERROR_SIZE, fmt, ap);
-  else if(!data->bits.mute) {
-    /* no errorbuffer receives this, write to data->err instead */
-    vfprintf(data->err, fmt, ap);
-    fprintf(data->err, "\n");
-  }
+  if(data->set.errorbuffer)
+    vsnprintf(data->set.errorbuffer, CURL_ERROR_SIZE, fmt, ap);
   va_end(ap);
 }
 
 /* Curl_sendf() sends formated data to the server */
 size_t Curl_sendf(int sockfd, struct connectdata *conn,
-                  char *fmt, ...)
+                  const char *fmt, ...)
 {
-  struct UrlData *data = conn->data;
+  struct SessionHandle *data = conn->data;
   size_t bytes_written;
   char *s;
   va_list ap;
@@ -163,8 +158,8 @@ size_t Curl_sendf(int sockfd, struct connectdata *conn,
   va_end(ap);
   if(!s)
     return 0; /* failure */
-  if(data->bits.verbose)
-    fprintf(data->err, "> %s", s);
+  if(data->set.verbose)
+    fprintf(data->set.err, "> %s", s);
 
   /* Write the buffer to the socket */
   Curl_write(conn, sockfd, s, strlen(s), &bytes_written);
@@ -186,21 +181,27 @@ CURLcode Curl_write(struct connectdata *conn, int sockfd,
   size_t bytes_written;
 
 #ifdef USE_SSLEAY
+  /* SSL_write() is said to return 'int' while write() and send() returns
+     'size_t' */
+  int ssl_bytes;
   if (conn->ssl.use) {
     int loop=100; /* just a precaution to never loop endlessly */
     while(loop--) {
-      bytes_written = SSL_write(conn->ssl.handle, mem, len);
-      if((-1 != bytes_written) ||
+      ssl_bytes = SSL_write(conn->ssl.handle, mem, len);
+      if((0 >= ssl_bytes) ||
          (SSL_ERROR_WANT_WRITE != SSL_get_error(conn->ssl.handle,
-                                                bytes_written) ))
+                                                ssl_bytes) )) {
+        /* this converts from signed to unsigned... */
+        bytes_written = ssl_bytes;
         break;
+      }
     }
   }
   else {
 #endif
 #ifdef KRB4
     if(conn->sec_complete) {
-      bytes_written = sec_write(conn, sockfd, mem, len);
+      bytes_written = Curl_sec_write(conn, sockfd, mem, len);
     }
     else
 #endif /* KRB4 */
@@ -218,7 +219,7 @@ CURLcode Curl_write(struct connectdata *conn, int sockfd,
    The bit pattern defines to what "streams" to write to. Body and/or header.
    The defines are in sendf.h of course.
  */
-CURLcode Curl_client_write(struct UrlData *data,
+CURLcode Curl_client_write(struct SessionHandle *data,
                            int type,
                            char *ptr,
                            size_t len)
@@ -229,21 +230,22 @@ CURLcode Curl_client_write(struct UrlData *data,
     len = strlen(ptr);
 
   if(type & CLIENTWRITE_BODY) {
-    wrote = data->fwrite(ptr, 1, len, data->out);
+    wrote = data->set.fwrite(ptr, 1, len, data->set.out);
     if(wrote != len) {
       failf (data, "Failed writing body");
       return CURLE_WRITE_ERROR;
     }
   }
-  if((type & CLIENTWRITE_HEADER) && data->writeheader) {
+  if((type & CLIENTWRITE_HEADER) &&
+     (data->set.fwrite_header || data->set.writeheader) ) {
     /*
      * Write headers to the same callback or to the especially setup
      * header callback function (added after version 7.7.1).
      */
     curl_write_callback writeit=
-      data->fwrite_header?data->fwrite_header:data->fwrite;
+      data->set.fwrite_header?data->set.fwrite_header:data->set.fwrite;
 
-    wrote = writeit(ptr, 1, len, data->writeheader);
+    wrote = writeit(ptr, 1, len, data->set.writeheader);
     if(wrote != len) {
       failf (data, "Failed writing header");
       return CURLE_WRITE_ERROR;
@@ -278,7 +280,7 @@ CURLcode Curl_read(struct connectdata *conn, int sockfd,
 #endif
 #ifdef KRB4
     if(conn->sec_complete)
-      nread = sec_read(conn, sockfd, buf, buffersize);
+      nread = Curl_sec_read(conn, sockfd, buf, buffersize);
     else
 #endif
       nread = sread (sockfd, buf, buffersize);
@@ -289,3 +291,11 @@ CURLcode Curl_read(struct connectdata *conn, int sockfd,
   return CURLE_OK;
 }
 
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
