@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.10 2001/01/25 12:21:10 bagder Exp $
+ * $Id: setup.h,v 1.13 2001/08/24 07:24:34 bagder Exp $
  *****************************************************************************/
 
 
@@ -32,7 +32,13 @@
 #endif
 
 #ifdef HAVE_CONFIG_H
+
+#ifdef VMS
+#include "config-vms.h"
+#else
 #include "config.h" /* the configure script results */
+#endif
+
 #else
 #ifdef WIN32
 /* include the hand-modified win32 adjusted config.h! */
@@ -44,6 +50,14 @@
 typedef char bool;
 #endif                     /* (rabe) */
 
+#ifdef NEED_REENTRANT
+/* Solaris machines needs _REENTRANT set for a few function prototypes and
+   things to appear in the #include files. We need to #define it before all
+   #include files */
+#define _REENTRANT
+#endif
+
+
 #include <stdio.h>
 #ifndef OS
 #ifdef WIN32
@@ -71,31 +85,17 @@ defined(HAVE_LIBSSL) && defined(HAVE_LIBCRYPTO)
 #endif
 
 #ifndef STDC_HEADERS /* no standard C headers! */
+#ifdef	VMS
+#include "../include/curl/stdcheaders.h"
+#else
 #include "curl/stdcheaders.h"
+#endif
 #else
 #ifdef _AIX
 #include "curl/stdcheaders.h"
 #endif
 #endif
 
-#if 0 /* zlib experiments are halted 17th october, 1999 (Daniel) */
-#if defined(HAVE_ZLIB_H) && defined(HAVE_LIBZ)
-     /* Both lib and header file exist, we have libz! */
-#define USE_ZLIB
-#endif
-#endif
-
-#if 0
-#ifdef HAVE_STRCASECMP
-#define strnequal(x,y,z) !(strncasecmp)(x,y,z)
-#define strequal(x,y) !(strcasecmp)(x,y)
-
-#else
-#define strnequal(x,y,z) !strnicmp(x,y,z)
-#define strequal(x,y) !stricmp(x,y)
-#endif
-#endif
-
 /* Below we define four functions. They should
    1. close a socket
    2. read from a socket
