@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: speedcheck.c,v 1.9 2001/03/09 15:14:51 bagder Exp $
+ * $Id: speedcheck.c,v 1.11 2001/09/07 04:01:32 bumblebury Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -34,37 +34,44 @@
 #include "sendf.h"
 #include "speedcheck.h"
 
-void Curl_speedinit(struct UrlData *data)
+void Curl_speedinit(struct SessionHandle *data)
 {
-  memset(&data->keeps_speed, 0, sizeof(struct timeval));
+  memset(&data->state.keeps_speed, 0, sizeof(struct timeval));
 }
 
-CURLcode Curl_speedcheck(struct UrlData *data,
+CURLcode Curl_speedcheck(struct SessionHandle *data,
                          struct timeval now)
 {
   if((data->progress.current_speed >= 0) &&
-     data->low_speed_time &&
-     (Curl_tvlong(data->keeps_speed) != 0) &&
-     (data->progress.current_speed < data->low_speed_limit)) {
+     data->set.low_speed_time &&
+     (Curl_tvlong(data->state.keeps_speed) != 0) &&
+     (data->progress.current_speed < data->set.low_speed_limit)) {
 
     /* We are now below the "low speed limit". If we are below it
        for "low speed time" seconds we consider that enough reason
        to abort the download. */
     
-    if( Curl_tvdiff(now, data->keeps_speed) > data->low_speed_time) {
+    if( Curl_tvdiff(now, data->state.keeps_speed) > data->set.low_speed_time) {
       /* we have been this slow for long enough, now die */
       failf(data,
 	    "Operation too slow. "
 	    "Less than %d bytes/sec transfered the last %d seconds",
-	    data->low_speed_limit,
-	    data->low_speed_time);
+	    data->set.low_speed_limit,
+	    data->set.low_speed_time);
       return CURLE_OPERATION_TIMEOUTED;
     }
   }
   else {
     /* we keep up the required speed all right */
-    data->keeps_speed = now;
+    data->state.keeps_speed = now;
   }
   return CURLE_OK;
 }
 
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
