@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: strequal.c,v 1.12 2001/03/14 08:47:56 bagder Exp $
+ * $Id: strequal.c,v 1.15 2001/09/07 04:01:33 bumblebury Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -28,11 +28,11 @@
 int curl_strequal(const char *first, const char *second)
 {
 #if defined(HAVE_STRCASECMP)
-  return !strcasecmp(first, second);
+  return !(strcasecmp)(first, second);
 #elif defined(HAVE_STRCMPI)
-  return !strcmpi(first, second);
+  return !(strcmpi)(first, second);
 #elif defined(HAVE_STRICMP)
-  return !stricmp(first, second);
+  return !(stricmp)(first, second);
 #else
   while (*first && *second) {
     if (toupper(*first) != toupper(*second)) {
@@ -80,7 +80,7 @@ int curl_strnequal(const char *first, const char *second, size_t max)
  *
  * 
  */
-size_t strlcat(char *dst, const char *src, size_t siz)
+size_t Curl_strlcat(char *dst, const char *src, size_t siz)
 {
   char *d = dst;
   const char *s = src;
@@ -107,3 +107,11 @@ size_t strlcat(char *dst, const char *src, size_t siz)
   return(dlen + (s - src));	/* count does not include NUL */
 }
 #endif
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
