@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: telnet.c,v 1.19 2001/03/16 15:19:36 bagder Exp $
+ * $Id: telnet.c,v 1.23 2001/09/17 14:10:38 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -35,7 +35,7 @@
 #include <errno.h>
 
 #if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
-#include <winsock.h>
+#include <winsock2.h>
 #include <time.h>
 #include <io.h>
 #else
@@ -105,7 +105,7 @@ void telrcv(struct connectdata *,
 	    unsigned char *inbuf,	/* Data received from socket */
 	    int count);			/* Number of bytes received */
 
-static void printoption(struct UrlData *data,
+static void printoption(struct SessionHandle *data,
 			const char *direction,
 			int cmd, int option);
 
@@ -114,7 +114,7 @@ static void send_negotiation(struct connectdata *, int cmd, int option);
 static void set_local_option(struct connectdata *, int cmd, int option);
 static void set_remote_option(struct connectdata *, int cmd, int option);
 
-static void printsub(struct UrlData *data,
+static void printsub(struct SessionHandle *data,
 		     int direction, unsigned char *pointer, int length);
 static void suboption(struct connectdata *);
 
@@ -152,8 +152,8 @@ struct TELNET {
   int him[256]; 
   int himq[256]; 
   int him_preferred[256]; 
-  char *subopt_ttype;             /* Set with suboption TTYPE */
-  char *subopt_xdisploc;          /* Set with suboption XDISPLOC */
+  char subopt_ttype[32];             /* Set with suboption TTYPE */
+  char subopt_xdisploc[128];          /* Set with suboption XDISPLOC */
   struct curl_slist *telnet_vars; /* Environment variables */
 
   /* suboptions */
@@ -215,13 +215,13 @@ static void negotiate(struct connectdata *conn)
   }
 }
 
-static void printoption(struct UrlData *data,
+static void printoption(struct SessionHandle *data,
 			const char *direction, int cmd, int option)
 {
-  char *fmt;
-  char *opt;
+  const char *fmt;
+  const char *opt;
    
-  if (data->bits.verbose)
+  if (data->set.verbose)
   {
     if (cmd == IAC)
     {
@@ -627,14 +627,14 @@ void rec_dont(struct connectdata *conn, int option)
 }
 
 
-static void printsub(struct UrlData *data,
+static void printsub(struct SessionHandle *data,
 		     int direction,		/* '<' or '>' */
 		     unsigned char *pointer,	/* where suboption data is */
 		     int length)		/* length of suboption data */
 {
   int i = 0;
 
-  if (data->bits.verbose)
+  if (data->set.verbose)
   {
     if (direction)
     {
@@ -745,7 +745,7 @@ static int check_telnet_options(struct connectdata *conn)
   char option_keyword[128];
   char option_arg[256];
   char *buf;
-  struct UrlData *data = conn->data;
+  struct SessionHandle *data = conn->data;
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
 
   /* Add the user name as an environment variable if it
@@ -753,26 +753,28 @@ static int check_telnet_options(struct connectdata *conn)
   if(conn->bits.user_passwd)
   {
     char *buf = malloc(256);
-    sprintf(buf, "USER,%s", data->user);
+    sprintf(buf, "USER,%s", data->state.user);
     tn->telnet_vars = curl_slist_append(tn->telnet_vars, buf);
 
     tn->us_preferred[TELOPT_NEW_ENVIRON] = YES;
   }
 
-  for(head = data->telnet_options; head; head=head->next) {
+  for(head = data->set.telnet_options; head; head=head->next) {
     if(sscanf(head->data, "%127[^= ]%*[ =]%255s",
               option_keyword, option_arg) == 2) {
 
       /* Terminal type */
       if(strequal(option_keyword, "TTYPE")) {
-        tn->subopt_ttype = option_arg;
+        strncpy(tn->subopt_ttype, option_arg, 31);
+        tn->subopt_ttype[31] = 0; /* String termination */
         tn->us_preferred[TELOPT_TTYPE] = YES;
         continue;
       }
 
       /* Display variable */
       if(strequal(option_keyword, "XDISPLOC")) {
-        tn->subopt_xdisploc = option_arg;
+        strncpy(tn->subopt_xdisploc, option_arg, 127);
+        tn->subopt_xdisploc[127] = 0; /* String termination */
         tn->us_preferred[TELOPT_XDISPLOC] = YES;
         continue;
       }
@@ -814,7 +816,7 @@ static void suboption(struct connectdata *conn)
   int tmplen;
   char varname[128];
   char varval[128];
-  struct UrlData *data = conn->data;
+  struct SessionHandle *data = conn->data;
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
 
   printsub(data, '<', (unsigned char *)tn->subbuffer, SB_LEN(tn)+2);
@@ -843,7 +845,7 @@ static void suboption(struct connectdata *conn)
       for(v = tn->telnet_vars;v;v = v->next) {
         tmplen = (strlen(v->data) + 1);
         /* Add the variable only if it fits */
-        if(len + tmplen < sizeof(temp)-6) {
+        if(len + tmplen < (int)sizeof(temp)-6) {
           sscanf(v->data, "%127[^,],%s", varname, varval);
           snprintf((char *)&temp[len], sizeof(temp) - len,
                    "%c%s%c%s", NEW_ENV_VAR, varname,
@@ -868,7 +870,7 @@ void telrcv(struct connectdata *conn,
 {
   unsigned char c;
   int index = 0;
-  struct UrlData *data = conn->data;
+  struct SessionHandle *data = conn->data;
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
 
   while(count--)
@@ -1031,13 +1033,20 @@ CURLcode Curl_telnet_done(struct connectdata *conn)
 CURLcode Curl_telnet(struct connectdata *conn)
 {
   CURLcode code;
-  struct UrlData *data = conn->data;
+  struct SessionHandle *data = conn->data;
   int sockfd = conn->firstsocket;
+#ifdef WIN32
+  WSAEVENT event_handle;
+  WSANETWORKEVENTS events;
+  HANDLE stdin_handle;
+  HANDLE objs[2];
+  DWORD waitret;
+#else
   fd_set readfd;
   fd_set keepfd;
-
+#endif
   bool keepon = TRUE;
-  char *buf = data->buffer;
+  char *buf = data->state.buffer;
   ssize_t nread;
   struct TELNET *tn;
 
@@ -1050,7 +1059,87 @@ CURLcode Curl_telnet(struct connectdata *conn)
   code = check_telnet_options(conn);
   if(code)
     return code;
-  
+
+#ifdef WIN32
+  /* We want to wait for both stdin and the socket. Since
+  ** the select() function in winsock only works on sockets
+  ** we have to use the WaitForMultipleObjects() call.
+  */
+
+  /* First, create a sockets event object */
+  event_handle = WSACreateEvent();
+
+  /* The get the Windows file handle for stdin */
+  stdin_handle = GetStdHandle(STD_INPUT_HANDLE);
+
+  /* Create the list of objects to wait for */
+  objs[0] = stdin_handle;
+  objs[1] = event_handle;
+
+  /* Tell winsock what events we want to listen to */
+  if(WSAEventSelect(sockfd, event_handle, FD_READ|FD_CLOSE) == SOCKET_ERROR) {
+    return 0;
+  }
+
+  /* Keep on listening and act on events */
+  while(keepon) {
+    waitret = WaitForMultipleObjects(2, objs, FALSE, INFINITE);
+    switch(waitret - WAIT_OBJECT_0)
+    {
+      case 0:
+      {
+        unsigned char outbuf[2];
+        int out_count = 0;
+        size_t bytes_written;
+        char *buffer = buf;
+              
+        if(!ReadFile(stdin_handle, buf, 255, &nread, NULL)) {
+          keepon = FALSE;
+          break;
+        }
+        
+        while(nread--) {
+          outbuf[0] = *buffer++;
+          out_count = 1;
+          if(outbuf[0] == IAC)
+            outbuf[out_count++] = IAC;
+          
+          Curl_write(conn, conn->firstsocket, outbuf,
+                     out_count, &bytes_written);
+        }
+      }
+      break;
+      
+      case 1:
+        if(WSAEnumNetworkEvents(sockfd, event_handle, &events)
+           != SOCKET_ERROR)
+        {
+          if(events.lNetworkEvents & FD_READ)
+          {
+            Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
+            
+            telrcv(conn, (unsigned char *)buf, nread);
+            
+            fflush(stdout);
+            
+            /* Negotiate if the peer has started negotiating,
+               otherwise don't. We don't want to speak telnet with
+               non-telnet servers, like POP or SMTP. */
+            if(tn->please_negotiate && !tn->already_negotiated) {
+              negotiate(conn);
+              tn->already_negotiated = 1;
+            }
+          }
+          
+          if(events.lNetworkEvents & FD_CLOSE)
+          {
+            keepon = FALSE;
+          }
+        }
+        break;
+    }
+  }
+#else
   FD_ZERO (&readfd);		/* clear it */
   FD_SET (sockfd, &readfd);
   FD_SET (1, &readfd);
@@ -1108,6 +1197,15 @@ CURLcode Curl_telnet(struct connectdata *conn)
       }
     }
   }
+#endif
   /* mark this as "no further transfer wanted" */
   return Curl_Transfer(conn, -1, -1, FALSE, NULL, -1, NULL);
 }
+
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
