@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: transfer.c,v 1.37 2001/05/03 10:45:39 bagder Exp $
+ * $Id: transfer.c,v 1.55 2001/09/12 12:02:12 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -90,6 +90,9 @@
 #include "progress.h"
 #include "getdate.h"
 #include "http.h"
+#include "url.h"
+#include "getinfo.h"
+#include "ssluse.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
@@ -103,10 +106,76 @@
 #define min(a, b)   ((a) < (b) ? (a) : (b))
 #endif
 
-/* Parts of this function was written by the friendly Mark Butler
-   <butlerm@xmission.com>. */
+/*
+ * compareheader()
+ *
+ * Returns TRUE if 'headerline' contains the 'header' with given 'content'.
+ * Pass headers WITH the colon.
+ */
+static bool
+compareheader(char *headerline, /* line to check */
+              const char *header,     /* header keyword _with_ colon */
+              const char *content)    /* content string to find */
+{
+  /* RFC2616, section 4.2 says: "Each header field consists of a name followed
+   * by a colon (":") and the field value. Field names are case-insensitive.
+   * The field value MAY be preceded by any amount of LWS, though a single SP
+   * is preferred." */
 
-CURLcode static
+  size_t hlen = strlen(header);
+  size_t clen;
+  size_t len;
+  char *start;
+  char *end;
+
+  if(!strnequal(headerline, header, hlen))
+    return FALSE; /* doesn't start with header */
+
+  /* pass the header */
+  start = &headerline[hlen];
+
+  /* pass all white spaces */
+  while(*start && isspace((int)*start))
+    start++;
+
+  /* find the end of the header line */
+  end = strchr(start, '\r'); /* lines end with CRLF */
+  if(!end) {
+    /* in case there's a non-standard compliant line here */
+    end = strchr(start, '\n');
+
+    if(!end)
+      /* hm, there's no line ending here, return false and bail out! */
+      return FALSE;
+  }
+
+  len = end-start; /* length of the content part of the input line */
+  clen = strlen(content); /* length of the word to find */
+
+  /* find the content string in the rest of the line */
+  for(;len>=clen;len--, start++) {
+    if(strnequal(start, content, clen))
+      return TRUE; /* match! */
+  }
+
+  return FALSE; /* no match */
+}
+
+/*
+ * Transfer()
+ *
+ * This function is what performs the actual transfer. It is capable of
+ * doing both ways simultaneously.
+ * The transfer must already have been setup by a call to Curl_Transfer().
+ *
+ * Note that headers are created in a preallocated buffer of a default size.
+ * That buffer can be enlarged on demand, but it is never shrinken again.
+ *
+ * Parts of this function was once written by the friendly Mark Butler
+ * <butlerm@xmission.com>.
+ */
+
+static CURLcode
 Transfer(struct connectdata *c_conn)
 {
   ssize_t nread;                /* number of bytes read */
@@ -128,7 +197,10 @@ Transfer(struct connectdata *c_conn)
   int offset = 0;		/* possible resume offset read from the
                                    Content-Range: header */
   int httpcode = 0;		/* error code from the 'HTTP/1.? XXX' line */
-  int httpversion = -1;         /* the last digit in the HTTP/1.1 string */
+  int httpversion = -1;			/* the HTTP version*10 */
+  bool write_after_100_header = FALSE;  /* should we enable the write after
+                                           we received a 100-continue/timeout
+                                           or directly */
 
   /* for the low speed checks: */
   CURLcode urg;
@@ -137,16 +209,16 @@ Transfer(struct connectdata *c_conn)
   int writetype;
 
   /* the highest fd we use + 1 */
-  struct UrlData *data;
+  struct SessionHandle *data;
   struct connectdata *conn = (struct connectdata *)c_conn;
   char *buf;
   int maxfd;
 
   data = conn->data; /* there's the root struct */
-  buf = data->buffer;
+  buf = data->state.buffer;
   maxfd = (conn->sockfd>conn->writesockfd?conn->sockfd:conn->writesockfd)+1;
 
-  hbufp = data->headerbuff;
+  hbufp = data->state.headerbuff;
 
   myalarm (0);			/* switch off the alarm-style timeout */
 
@@ -172,7 +244,7 @@ Transfer(struct connectdata *c_conn)
   }
   /* we want header and/or body, if neither then don't do this! */
   if(conn->getheader ||
-     !data->bits.no_body) {
+     !data->set.no_body) {
     fd_set readfd;
     fd_set writefd;
     fd_set rkeepfd;
@@ -195,8 +267,13 @@ Transfer(struct connectdata *c_conn)
 
     FD_ZERO (&writefd);		/* clear it */
     if(conn->writesockfd != -1) {
-      FD_SET (conn->writesockfd, &writefd); /* write socket */
-      keepon |= KEEP_WRITE;
+      if (data->set.expect100header)
+        /* wait with write until we either got 100-continue or a timeout */
+        write_after_100_header = TRUE;
+      else {
+        FD_SET (conn->writesockfd, &writefd); /* write socket */
+        keepon |= KEEP_WRITE;
+      }
     }
 
     /* get these in backup variables to be able to restore them on each lap in
@@ -222,6 +299,12 @@ Transfer(struct connectdata *c_conn)
           keepon = 0; /* no more read or write */
 	continue;
       case 0:			/* timeout */
+        if (write_after_100_header) {
+          write_after_100_header = FALSE;
+          FD_SET (conn->writesockfd, &writefd); /* write socket */
+          keepon |= KEEP_WRITE;
+          wkeepfd = writefd;
+        }
 	break;
       default:
         if((keepon & KEEP_READ) && FD_ISSET(conn->sockfd, &readfd)) {
@@ -236,6 +319,7 @@ Transfer(struct connectdata *c_conn)
              we bail out from this! */
           else if (0 >= (signed int) nread) {
             keepon &= ~KEEP_READ;
+            FD_ZERO(&rkeepfd);
             break;
           }
 
@@ -261,20 +345,24 @@ Transfer(struct connectdata *c_conn)
                 /* no more complete header lines within buffer */
                 /* copy what is remaining into headerbuff */
                 int str_length = (int)strlen(str);
-                
-                if (hbuflen + (int)str_length >= data->headersize) {
+
+                /*
+                 * We enlarge the header buffer if it seems to be too
+                 * smallish
+                 */
+                if (hbuflen + (int)str_length >= data->state.headersize) {
                   char *newbuff;
                   long newsize=MAX((hbuflen+str_length)*3/2,
-                                   data->headersize*2);
-                  hbufp_index = hbufp - data->headerbuff;
-                  newbuff = (char *)realloc(data->headerbuff, newsize);
+                                   data->state.headersize*2);
+                  hbufp_index = hbufp - data->state.headerbuff;
+                  newbuff = (char *)realloc(data->state.headerbuff, newsize);
                   if(!newbuff) {
                     failf (data, "Failed to alloc memory for big header!");
                     return CURLE_READ_ERROR;
                   }
-                  data->headersize=newsize;
-                  data->headerbuff = newbuff;
-                  hbufp = data->headerbuff + hbufp_index;
+                  data->state.headersize=newsize;
+                  data->state.headerbuff = newbuff;
+                  hbufp = data->state.headerbuff + hbufp_index;
                 }
                 strcpy (hbufp, str);
                 hbufp += strlen (str);
@@ -284,19 +372,25 @@ Transfer(struct connectdata *c_conn)
 
               str = end_ptr + 1;	/* move just past new line */
 
-              if (hbuflen + (str - str_start) >= data->headersize) {
+              /*
+               * We're about to copy a chunk of data to the end of the
+               * already received header. We make sure that the full string
+               * fit in the allocated header buffer, or else we enlarge 
+               * it.
+               */
+              if (hbuflen + (str - str_start) >= data->state.headersize) {
                 char *newbuff;
                 long newsize=MAX((hbuflen+(str-str_start))*3/2,
-                                 data->headersize*2);
-                hbufp_index = hbufp - data->headerbuff;
-                newbuff = (char *)realloc(data->headerbuff, newsize);
+                                 data->state.headersize*2);
+                hbufp_index = hbufp - data->state.headerbuff;
+                newbuff = (char *)realloc(data->state.headerbuff, newsize);
                 if(!newbuff) {
                   failf (data, "Failed to alloc memory for big header!");
                   return CURLE_READ_ERROR;
                 }
-                data->headersize= newsize;
-                data->headerbuff = newbuff;
-                hbufp = data->headerbuff + hbufp_index;
+                data->state.headersize= newsize;
+                data->state.headerbuff = newbuff;
+                hbufp = data->state.headerbuff + hbufp_index;
               }
 
               /* copy to end of line */
@@ -305,25 +399,19 @@ Transfer(struct connectdata *c_conn)
               hbuflen += str - str_start;
               *hbufp = 0;
               
-              p = data->headerbuff;
+              p = data->state.headerbuff;
               
-              /* we now have a full line that p points to */
-              if (('\n' == *p) || ('\r' == *p)) {
-                /* Zero-length line means end of header! */
-#if 0
-                if (-1 != conn->size)	/* if known */
-                  conn->size += bytecount; /* we append the already read
-                                              size */
-#endif
+              /****
+               * We now have a FULL header line that p points to
+               *****/
 
+              if (('\n' == *p) || ('\r' == *p)) {
+                /* Zero-length header line means end of headers! */
 
                 if ('\r' == *p)
                   p++;		/* pass the \r byte */
                 if ('\n' == *p)
                   p++;		/* pass the \n byte */
-#if 0 /* headers are not included in the size */
-                Curl_pgrsSetDownloadSize(data, conn->size);
-#endif
 
                 if(100 == httpcode) {
                   /*
@@ -335,31 +423,50 @@ Transfer(struct connectdata *c_conn)
                    */
                   header = TRUE;
                   headerline = 0; /* we restart the header line counter */
+                  /* if we did wait for this do enable write now! */
+                  if (write_after_100_header) {
+                    write_after_100_header = FALSE;
+                    FD_SET (conn->writesockfd, &writefd); /* write socket */
+                    keepon |= KEEP_WRITE;
+                    wkeepfd = writefd;
+                  }
                 }
-                else
+		else
                   header = FALSE;	/* no more header to parse! */
 
+		if (417 == httpcode) {
+		  /*
+		   * we got: "417 Expectation Failed" this means:
+		   * we have made a HTTP call and our Expect Header
+		   * seems to cause a problem => abort the write operations
+		   * (or prevent them from starting
+		   */
+		  write_after_100_header = FALSE;
+		  keepon &= ~KEEP_WRITE;
+		  FD_ZERO(&wkeepfd);
+		}
+
                 /* now, only output this if the header AND body are requested:
                  */
                 writetype = CLIENTWRITE_HEADER;
-                if (data->bits.http_include_header)
+                if (data->set.http_include_header)
                   writetype |= CLIENTWRITE_BODY;
 
-                urg = Curl_client_write(data, writetype, data->headerbuff,
-                                        p - data->headerbuff);
+                urg = Curl_client_write(data, writetype, data->state.headerbuff,
+                                        p - data->state.headerbuff);
                 if(urg)
                   return urg;
 
-                data->header_size += p - data->headerbuff;
+                data->info.header_size += p - data->state.headerbuff;
 
                 if(!header) {
                   /*
-                   * end-of-headers.
+                   * really end-of-headers.
                    *
                    * If we requested a "no body", this is a good time to get
                    * out and return home.
                    */
-                  if(data->bits.no_body)
+                  if(data->set.no_body)
                     return CURLE_OK;
 
                   if(!conn->bits.close) {
@@ -382,34 +489,52 @@ Transfer(struct connectdata *c_conn)
 
                 /* We continue reading headers, so reset the line-based
                    header parsing variables hbufp && hbuflen */
-                hbufp = data->headerbuff;
+                hbufp = data->state.headerbuff;
                 hbuflen = 0;
                 continue;
               }
+
+              /*
+               * Checks for special headers coming up.
+               */
               
               if (!headerline++) {
                 /* This is the first header, it MUST be the error code line
                    or else we consiser this to be the body right away! */
-                if (2 == sscanf (p, " HTTP/1.%d %3d", &httpversion,
-                                 &httpcode)) {
+                int httpversion_major;
+                int nc=sscanf (p, " HTTP/%d.%d %3d",
+                               &httpversion_major ,&httpversion, &httpcode);
+                if (nc==3) {
+                  httpversion+=10*httpversion_major;
+                }
+                else {
+                  /* this is the real world, not a Nirvana
+                     NCSA 1.5.x returns this crap when asked for HTTP/1.1
+                  */
+                  nc=sscanf (p, " HTTP %3d", &httpcode);
+                  httpversion = 10;
+                }
+
+                if (nc) {
+                  data->info.httpcode = httpcode;
+                  data->info.httpversion = httpversion;
+
                   /* 404 -> URL not found! */
                   if (
-                      ( ((data->bits.http_follow_location) &&
+                      ( ((data->set.http_follow_location) &&
                          (httpcode >= 400))
                         ||
-                        (!data->bits.http_follow_location &&
+                        (!data->set.http_follow_location &&
                          (httpcode >= 300)))
-                      && (data->bits.http_fail_on_error)) {
+                      && (data->set.http_fail_on_error)) {
                     /* If we have been told to fail hard on HTTP-errors,
                        here is the check for that: */
                     /* serious error, go home! */
                     failf (data, "The requested file was not found");
                     return CURLE_HTTP_NOT_FOUND;
                   }
-                  data->progress.httpcode = httpcode;
-                  data->progress.httpversion = httpversion;
 
-                  if(httpversion == 0)
+                  if(httpversion == 10)
                     /* Default action for HTTP/1.0 must be to close, unless
                        we get one of those fancy headers that tell us the
                        server keeps it open for us! */
@@ -430,15 +555,14 @@ Transfer(struct connectdata *c_conn)
                 }
               }
               /* check for Content-Length: header lines to get size */
-              if (strnequal("Content-Length", p, 14) &&
-                  sscanf (p+14, ": %ld", &contentlength)) {
+              if (strnequal("Content-Length:", p, 15) &&
+                  sscanf (p+15, " %ld", &contentlength)) {
                 conn->size = contentlength;
                 Curl_pgrsSetDownloadSize(data, contentlength);
               }
-              else if((httpversion == 0) &&
+              else if((httpversion == 10) &&
                       conn->bits.httpproxy &&
-                      strnequal("Proxy-Connection: keep-alive", p,
-                                strlen("Proxy-Connection: keep-alive"))) {
+                      compareheader(p, "Proxy-Connection:", "keep-alive")) {
                 /*
                  * When a HTTP/1.0 reply comes when using a proxy, the
                  * 'Proxy-Connection: keep-alive' line tells us the
@@ -448,9 +572,8 @@ Transfer(struct connectdata *c_conn)
                 conn->bits.close = FALSE; /* don't close when done */
                 infof(data, "HTTP/1.0 proxy connection set to keep alive!\n");
               }
-              else if((httpversion == 0) &&
-                      strnequal("Connection: keep-alive", p,
-                                strlen("Connection: keep-alive"))) {
+              else if((httpversion == 10) &&
+                      compareheader(p, "Connection:", "keep-alive")) {
                 /*
                  * A HTTP/1.0 reply with the 'Connection: keep-alive' line
                  * tells us the connection will be kept alive for our
@@ -460,8 +583,7 @@ Transfer(struct connectdata *c_conn)
                 conn->bits.close = FALSE; /* don't close when done */
                 infof(data, "HTTP/1.0 connection set to keep alive!\n");
               }
-              else if (strnequal("Connection: close", p,
-                                 strlen("Connection: close"))) {
+              else if (compareheader(p, "Connection:", "close")) {
                 /*
                  * [RFC 2616, section 8.1.2.1]
                  * "Connection: close" is HTTP/1.1 language and means that
@@ -470,8 +592,7 @@ Transfer(struct connectdata *c_conn)
                  */
                 conn->bits.close = TRUE; /* close when done */
               }
-              else if (strnequal("Transfer-Encoding: chunked", p,
-                                 strlen("Transfer-Encoding: chunked"))) {
+              else if (compareheader(p, "Transfer-Encoding:", "chunked")) {
                 /*
                  * [RFC 2616, section 3.6.1] A 'chunked' transfer encoding
                  * means that the server will send a series of "chunks". Each
@@ -485,9 +606,9 @@ Transfer(struct connectdata *c_conn)
                 /* init our chunky engine */
                 Curl_httpchunk_init(conn);
               }
-              else if (strnequal("Content-Range", p, 13)) {
-                if (sscanf (p+13, ": bytes %d-", &offset) ||
-                    sscanf (p+13, ": bytes: %d-", &offset)) {
+              else if (strnequal("Content-Range:", p, 14)) {
+                if (sscanf (p+14, " bytes %d-", &offset) ||
+                    sscanf (p+14, " bytes: %d-", &offset)) {
                   /* This second format was added August 1st 2000 by Igor
                      Khristophorov since Sun's webserver JavaWebServer/1.1.1
                      obviously sends the header this way! :-( */
@@ -498,27 +619,33 @@ Transfer(struct connectdata *c_conn)
                 }
               }
               else if(data->cookies &&
-                      strnequal("Set-Cookie: ", p, 11)) {
+                      strnequal("Set-Cookie:", p, 11)) {
                 Curl_cookie_add(data->cookies, TRUE, &p[12]);
               }
               else if(strnequal("Last-Modified:", p,
                                 strlen("Last-Modified:")) &&
-                      (data->timecondition || data->bits.get_filetime) ) {
+                      (data->set.timecondition || data->set.get_filetime) ) {
                 time_t secs=time(NULL);
                 timeofdoc = curl_getdate(p+strlen("Last-Modified:"), &secs);
-                if(data->bits.get_filetime)
-                  data->progress.filetime = timeofdoc;
+                if(data->set.get_filetime)
+                  data->info.filetime = timeofdoc;
               }
               else if ((httpcode >= 300 && httpcode < 400) &&
-                       (data->bits.http_follow_location) &&
-                       strnequal("Location: ", p, 10)) {
+                       (data->set.http_follow_location) &&
+                       strnequal("Location:", p, 9)) {
                 /* this is the URL that the server advices us to get instead */
                 char *ptr;
                 char *start=p;
                 char backup;
 
-                start += 10; /* pass "Location: " */
+                start += 9; /* pass "Location:" */
+
+                /* Skip spaces and tabs. We do this to support multiple
+                   white spaces after the "Location:" keyword. */
+                while(*start && isspace((int)*start ))
+                  start++;
                 ptr = start; /* start scanning here */
+
                 /* scan through the string to find the end */
                 while(*ptr && !isspace((int)*ptr))
                   ptr++;
@@ -528,18 +655,22 @@ Transfer(struct connectdata *c_conn)
                 *ptr = backup; /* restore ending letter */
               }
 
+              /*
+               * End of header-checks. Write them to the client.
+               */
+
               writetype = CLIENTWRITE_HEADER;
-              if (data->bits.http_include_header)
+              if (data->set.http_include_header)
                 writetype |= CLIENTWRITE_BODY;
 
               urg = Curl_client_write(data, writetype, p, hbuflen);
               if(urg)
                 return urg;
 
-              data->header_size += hbuflen;
+              data->info.header_size += hbuflen;
               
               /* reset hbufp pointer && hbuflen */
-              hbufp = data->headerbuff;
+              hbufp = data->state.headerbuff;
               hbuflen = 0;
             }
             while (*str);		/* header line within buffer */
@@ -575,7 +706,7 @@ Transfer(struct connectdata *c_conn)
                 }
                 else if (conn->resume_from &&
                          !content_range &&
-                         (data->httpreq==HTTPREQ_GET)) {
+                         (data->set.httpreq==HTTPREQ_GET)) {
                   /* we wanted to resume a download, although the server
                      doesn't seem to support this and we did this with a GET
                      (if it wasn't a GET we did a POST or PUT resume) */
@@ -583,23 +714,23 @@ Transfer(struct connectdata *c_conn)
                          "byte ranges. Cannot resume.");
                   return CURLE_HTTP_RANGE_ERROR;
                 }
-                else if(data->timecondition && !conn->range) {
+                else if(data->set.timecondition && !conn->range) {
                   /* A time condition has been set AND no ranges have been
                      requested. This seems to be what chapter 13.3.4 of
                      RFC 2616 defines to be the correct action for a
                      HTTP/1.1 client */
-                  if((timeofdoc > 0) && (data->timevalue > 0)) {
-                    switch(data->timecondition) {
+                  if((timeofdoc > 0) && (data->set.timevalue > 0)) {
+                    switch(data->set.timecondition) {
                     case TIMECOND_IFMODSINCE:
                     default:
-                      if(timeofdoc < data->timevalue) {
+                      if(timeofdoc < data->set.timevalue) {
                         infof(data,
                               "The requested document is not new enough\n");
                         return CURLE_OK;
                       }
                       break;
                     case TIMECOND_IFUNMODSINCE:
-                      if(timeofdoc > data->timevalue) {
+                      if(timeofdoc > data->set.timevalue) {
                         infof(data,
                               "The requested document is not old enough\n");
                         return CURLE_OK;
@@ -630,6 +761,7 @@ Transfer(struct connectdata *c_conn)
               else if(CHUNKE_STOP == res) {
                 /* we're done reading chunks! */
                 keepon &= ~KEEP_READ; /* read no more */
+                FD_ZERO(&rkeepfd);
 
                 /* There are now possibly N number of bytes at the end of the
                    str buffer that weren't written to the client, but we don't
@@ -643,7 +775,9 @@ Transfer(struct connectdata *c_conn)
               nread = conn->maxdownload - bytecount;
               if((signed int)nread < 0 ) /* this should be unusual */
                 nread = 0;
+
               keepon &= ~KEEP_READ; /* we're done reading */
+              FD_ZERO(&rkeepfd);
             }
 
             bytecount += nread;
@@ -663,48 +797,47 @@ Transfer(struct connectdata *c_conn)
         if((keepon & KEEP_WRITE) && FD_ISSET(conn->writesockfd, &writefd)) {
           /* write */
 
-          char scratch[BUFSIZE * 2];
           int i, si;
           size_t bytes_written;
 
-          if(data->crlf)
-            buf = data->buffer; /* put it back on the buffer */
-
-          nread = data->fread(buf, 1, conn->upload_bufsize, data->in);
+          nread = data->set.fread(buf, 1, conn->upload_bufsize, data->set.in);
 
           /* the signed int typecase of nread of for systems that has
              unsigned size_t */
           if ((signed int)nread<=0) {
             /* done */
             keepon &= ~KEEP_WRITE; /* we're done writing */
+            FD_ZERO(&wkeepfd);
             break;
           }
           writebytecount += nread;
           Curl_pgrsSetUploadCounter(data, (double)writebytecount);            
 
           /* convert LF to CRLF if so asked */
-          if (data->crlf) {
+          if (data->set.crlf) {
             for(i = 0, si = 0; i < (int)nread; i++, si++) {
               if (buf[i] == 0x0a) {
-                scratch[si++] = 0x0d;
-                scratch[si] = 0x0a;
+                data->state.scratch[si++] = 0x0d;
+                data->state.scratch[si] = 0x0a;
               }
               else {
-                scratch[si] = buf[i];
+                data->state.scratch[si] = buf[i];
               }
             }
             nread = si;
-            buf = scratch; /* point to the new buffer */
+            buf = data->state.scratch; /* point to the new buffer */
           }
 
           /* write to socket */
           urg = Curl_write(conn, conn->writesockfd, buf, nread,
                            &bytes_written);
 
-          if(nread != bytes_written) {
+          if(nread != (int)bytes_written) {
             failf(data, "Failed uploading data");
             return CURLE_WRITE_ERROR;
           }
+          if(data->set.crlf)
+            buf = data->state.buffer; /* put it back on the buffer */
 
         }
 
@@ -728,14 +861,20 @@ Transfer(struct connectdata *c_conn)
         conn->upload_bufsize=(long)min(data->progress.ulspeed, BUFSIZE);
       }
 
-      if (data->timeout && (Curl_tvdiff (now, start) > data->timeout)) {
+      if (data->set.timeout && (Curl_tvdiff (now, start) > data->set.timeout)) {
 	failf (data, "Operation timed out with %d out of %d bytes received",
 	       bytecount, conn->size);
 	return CURLE_OPERATION_TIMEOUTED;
       }
     }
   }
-  if(!(data->bits.no_body) && contentlength &&
+
+  /*
+   * The tranfer has been performed. Just make some general checks before
+   * returning.
+   */
+
+  if(!(data->set.no_body) && contentlength &&
      (bytecount != contentlength)) {
     failf(data, "transfer closed with %d bytes remaining to read",
           contentlength-bytecount);
@@ -757,20 +896,28 @@ Transfer(struct connectdata *c_conn)
   return CURLE_OK;
 }
 
-CURLcode Curl_perform(CURL *curl)
+CURLcode Curl_perform(struct SessionHandle *data)
 {
   CURLcode res;
-  struct UrlData *data = (struct UrlData *)curl;
   struct connectdata *conn=NULL;
-  bool port=TRUE; /* allow data->use_port to set port to use */
+  bool port=TRUE; /* allow data->set.use_port to set port to use */
   char *newurl = NULL; /* possibly a new URL to follow to! */
 
-  if(!data->url)
+  if(!data->change.url)
     /* we can't do anything wihout URL */
     return CURLE_URL_MALFORMAT;
 
-  data->followlocation=0; /* reset the location-follow counter */
-  data->bits.this_is_a_follow = FALSE; /* reset this */
+#ifdef USE_SSLEAY
+  /* Init the SSL session ID cache here. We do it here since we want to
+     do it after the *_setopt() calls (that could change the size) but
+     before any transfer. */
+  Curl_SSL_InitSessions(data, data->set.ssl.numsessions);
+#endif
+
+  data->set.followlocation=0; /* reset the location-follow counter */
+  data->state.this_is_a_follow = FALSE; /* reset this */
+
+  Curl_initinfo(data); /* reset session-specific information "variables" */
 
   Curl_pgrsStartNow(data);
 
@@ -780,7 +927,14 @@ CURLcode Curl_perform(CURL *curl)
     if(res == CURLE_OK) {
       res = Curl_do(conn);
       if(res == CURLE_OK) {
+        if(conn->protocol&PROT_FTPS)
+          /* FTPS, disable ssl while transfering data */
+          conn->ssl.use = FALSE;
         res = Transfer(conn); /* now fetch that URL please */
+        if(conn->protocol&PROT_FTPS)
+          /* FTPS, enable ssl again after havving transferred data */
+          conn->ssl.use = TRUE;
+
         if(res == CURLE_OK) {
           /*
            * We must duplicate the new URL here as the connection data
@@ -808,30 +962,28 @@ CURLcode Curl_perform(CURL *curl)
         port=TRUE; /* by default we use the user set port number even after
                       a Location: */
 
-	if (data->maxredirs && (data->followlocation >= data->maxredirs)) {
-	  failf(data,"Maximum (%d) redirects followed", data->maxredirs);
+	if (data->set.maxredirs && (data->set.followlocation >= data->set.maxredirs)) {
+	  failf(data,"Maximum (%d) redirects followed", data->set.maxredirs);
           res=CURLE_TOO_MANY_REDIRECTS;
 	  break;
 	}
 
         /* mark the next request as a followed location: */
-        data->bits.this_is_a_follow = TRUE;
+        data->state.this_is_a_follow = TRUE;
 
-        data->followlocation++; /* count location-followers */
+        data->set.followlocation++; /* count location-followers */
 
-        if(data->bits.http_auto_referer) {
+        if(data->set.http_auto_referer) {
           /* We are asked to automatically set the previous URL as the
              referer when we get the next URL. We pick the ->url field,
              which may or may not be 100% correct */
 
-          if(data->free_referer) {
+          if(data->change.referer_alloc)
             /* If we already have an allocated referer, free this first */
-            free(data->referer);
-          }
+            free(data->change.referer);
 
-          data->referer = strdup(data->url);
-          data->free_referer = TRUE;          /* yes, free this later */
-          data->bits.http_set_referer = TRUE; /* might have been false */
+          data->change.referer = strdup(data->change.url);
+          data->change.referer_alloc = TRUE; /* yes, free this later */
         }
 
         if(2 != sscanf(newurl, "%15[^:]://%c", prot, &letter)) {
@@ -849,7 +1001,7 @@ CURLcode Curl_perform(CURL *curl)
 
           /* we must make our own copy of the URL to play with, as it may
              point to read-only data */
-          char *url_clone=strdup(data->url);
+          char *url_clone=strdup(data->change.url);
 
           if(!url_clone)
             return CURLE_OUT_OF_MEMORY;
@@ -899,15 +1051,16 @@ CURLcode Curl_perform(CURL *curl)
           port = FALSE;
         }
 
-        if(data->bits.urlstringalloc)
-          free(data->url);
+        if(data->change.url_alloc)
+          free(data->change.url);
+        else
+          data->change.url_alloc = TRUE; /* the URL is allocated */
       
         /* TBD: set the URL with curl_setopt() */
-        data->url = newurl;
+        data->change.url = newurl;
+        newurl = NULL; /* don't free! */
 
-        data->bits.urlstringalloc = TRUE; /* the URL is allocated */
-
-        infof(data, "Follows Location: to new URL: '%s'\n", data->url);
+        infof(data, "Follows Location: to new URL: '%s'\n", data->change.url);
 
         /*
          * We get here when the HTTP code is 300-399. We need to perform
@@ -915,7 +1068,7 @@ CURLcode Curl_perform(CURL *curl)
          * Discussed on the curl mailing list and posted about on the 26th
          * of January 2001.
          */
-        switch(data->progress.httpcode) {
+        switch(data->info.httpcode) {
         case 300: /* Multiple Choices */
         case 301: /* Moved Permanently */
         case 306: /* Not used */
@@ -946,10 +1099,8 @@ CURLcode Curl_perform(CURL *curl)
         case 303: /* See Other */
           /* Disable both types of POSTs, since doing a second POST when
            * following isn't what anyone would want! */
-          data->bits.http_post = FALSE;
-          data->bits.http_formpost = FALSE;
-          data->httpreq = HTTPREQ_GET; /* enfore GET request */
-          infof(data, "Disables POST\n");
+          data->set.httpreq = HTTPREQ_GET; /* enforce GET request */
+          infof(data, "Disables POST, goes with GET\n");
           break;
         case 304: /* Not Modified */
           /* 304 means we did a conditional request and it was "Not modified".
@@ -977,13 +1128,12 @@ CURLcode Curl_perform(CURL *curl)
     free(newurl);
 
   /* make sure the alarm is switched off! */
-  if(data->timeout || data->connecttimeout)
+  if(data->set.timeout || data->set.connecttimeout)
     myalarm(0);
 
   return res;
 }
 
-
 CURLcode 
 Curl_Transfer(struct connectdata *c_conn, /* connection data */
               int sockfd,	/* socket to read from or -1 */
@@ -1012,3 +1162,10 @@ Curl_Transfer(struct connectdata *c_conn, /* connection data */
 
 }
           
+/*
+ * local variables:
+ * eval: (load-file "../curl-mode.el")
+ * end:
+ * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
+ * vim<600: et sw=2 ts=2 sts=2 tw=78
+ */
