@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 2001, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * In order to be useful for every potential user, curl and libcurl are
  * dual-licensed under the MPL and the MIT/X-derivate licenses.
@@ -20,17 +20,13 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: urldata.h,v 1.61 2001/05/04 07:47:11 bagder Exp $
+ * $Id: urldata.h,v 1.78 2001/09/12 12:02:12 bagder Exp $
  *****************************************************************************/
 
 /* This file is for lib internal stuff */
 
 #include "setup.h"
 
-#ifndef MAXHOSTNAMELEN
-#define MAXHOSTNAMELEN 256
-#endif
-
 #define PORT_FTP 21
 #define PORT_TELNET 23
 #define PORT_GOPHER 70
@@ -125,14 +121,26 @@ struct ssl_connect_data {
 #endif /* USE_SSLEAY */
 };
 
+/* information about one single SSL session */
+struct curl_ssl_session {
+  char *name;       /* host name for which this ID was used */
+  void *sessionid;  /* as returned from the SSL layer */
+  long age;         /* just a number, the higher the more recent */
+  unsigned short remote_port; /* remote port to connect to */
+};
+
 struct ssl_config_data {
   long version;          /* what version the client wants to use */
   long certverifyresult; /* result from the certificate verification */
   long verifypeer;       /* set TRUE if this is desired */
+  long verifyhost;       /* 0: no verif, 1: check that CN exists, 2: CN must match hostname */
   char *CApath;          /* DOES NOT WORK ON WINDOWS */
   char *CAfile;          /* cerficate to verify peer against */
   char *random_file;     /* path to file containing "random" data */
   char *egdsocket;       /* path to file containing the EGD daemon socket */
+  char *cipher_list;     /* list of ciphers to use */
+
+  long numsessions;                 /* SSL session id cache size */
 };
 
 /****************************************************************************
@@ -141,14 +149,14 @@ struct ssl_config_data {
 struct HTTP {
   struct FormData *sendit;
   int postsize;
-  char *p_pragma;      /* Pragma: string */
-  char *p_accept;      /* Accept: string */
+  const char *p_pragma;      /* Pragma: string */
+  const char *p_accept;      /* Accept: string */
   long readbytecount; 
   long writebytecount;
 
   /* For FORM posting */
   struct Form form;
-  size_t (*storefread)(char *, size_t , size_t , FILE *);
+  curl_read_callback storefread;
   FILE *in;
 
   struct Curl_chunker chunk;
@@ -182,12 +190,15 @@ struct ConnectBits {
   bool close; /* if set, we close the connection after this request */
   bool reuse; /* if set, this is a re-used connection */
   bool chunk; /* if set, this is a chunked transfer-encoding */
-  bool httpproxy; /* if set, this transfer is done through a http proxy */
-  bool user_passwd;       /* do we use user+password for this connection? */
+  bool httpproxy;    /* if set, this transfer is done through a http proxy */
+  bool user_passwd;    /* do we use user+password for this connection? */
   bool proxy_user_passwd; /* user+password for the proxy? */
 
   bool use_range;
   bool rangestringalloc; /* the range string is malloc()'ed */
+
+  bool resume_done; /* nothing was transfered, resumed transfer already
+                       complete */
 };
 
 /*
@@ -196,7 +207,7 @@ struct ConnectBits {
  */
 struct connectdata {
   /**** Fields set when inited and not modified again */
-  struct UrlData *data; /* link to the root CURL struct */
+  struct SessionHandle *data; /* link to the root CURL struct */
   int connectindex; /* what index in the connects index this particular
                        struct has */
 
@@ -311,6 +322,9 @@ struct connectdata {
   int sec_complete;
   void *app_data;
 
+  struct Curl_sec_client_mech *mech;
+  struct sockaddr_in local_addr;
+
 #endif
 
   /*************** Request - specific items ************/
@@ -329,9 +343,21 @@ struct connectdata {
     void *generic;
   } proto;
 
-
 };
 
+/*
+ * Struct to keep statistical and informational data.
+ */
+struct PureInfo {
+  int httpcode;
+  int httpversion;
+  time_t filetime; /* If requested, this is might get set. It may be 0 if
+                      the time was unretrievable */
+  long header_size;  /* size of read header(s) in bytes */
+  long request_size; /* the amount of bytes sent in the request(s) */
+};
+
+
 struct Progress {
   long lastshow; /* time() of the last displayed progress meter or NULL to
                     force redraw at next call */
@@ -349,17 +375,12 @@ struct Progress {
   double dlspeed;
   double ulspeed;
 
-  struct timeval start;
-  struct timeval t_startsingle;
-  /* various data stored for possible later report */
   double t_nslookup;
   double t_connect;
   double t_pretransfer;
-  int httpcode;
-  int httpversion;
-  time_t filetime; /* If requested, this is might get set. It may be 0 if
-                      the time was unretrievable */
 
+  struct timeval start;
+  struct timeval t_startsingle;
 #define CURR_TIME 5
 
   double speeder[ CURR_TIME ];
@@ -376,14 +397,143 @@ typedef enum {
   HTTPREQ_LAST /* last in list */
 } Curl_HttpReq;
 
-/* This struct is for boolean settings that define how to behave during
-   this session. */
-struct Configbits {
-  /* these four request types mirror the httpreq field */
-  bool http_formpost;
-  bool http_post;
-  bool http_put;
-  bool http_get;
+/*
+ * Values that are generated, temporary or calculated internally for a
+ * "session handle" must be defined within the 'struct urlstate'.  This struct
+ * will be used within the SessionHandle struct. When the 'SessionHandle'
+ * struct is cloned, this data MUST NOT be copied.
+ *
+ * Remember that any "state" information goes globally for the curl handle.
+ * Session-data MUST be put in the connectdata struct and here.  */
+#define MAX_CURL_USER_LENGTH 256
+#define MAX_CURL_PASSWORD_LENGTH 256
+
+struct UrlState {
+  /* buffers to store authentication data in, as parsed from input options */
+  char user[MAX_CURL_USER_LENGTH];
+  char passwd[MAX_CURL_PASSWORD_LENGTH];
+  char proxyuser[MAX_CURL_USER_LENGTH];
+  char proxypasswd[MAX_CURL_PASSWORD_LENGTH];
+
+  struct timeval keeps_speed; /* for the progress meter really */
+
+  /* 'connects' will be an allocated array with pointers. If the pointer is
+     set, it holds an allocated connection. */
+  struct connectdata **connects;
+  long numconnects; /* size of the 'connects' array */
+
+  char *headerbuff; /* allocated buffer to store headers in */
+  int headersize;   /* size of the allocation */
+
+  char buffer[BUFSIZE+1]; /* buffer with size BUFSIZE */
+
+  double current_speed;  /* the ProgressShow() funcion sets this */
+
+  bool this_is_a_follow; /* this is a followed Location: request */
+
+  char *auth_host; /* if set, this should be the host name that we will
+                      sent authorization to, no else. Used to make Location:
+                      following not keep sending user+password... This is
+                      strdup() data.
+                    */
+
+  struct curl_ssl_session *session; /* array of 'numsessions' size */
+  long sessionage;                  /* number of the most recent session */
+
+  char scratch[BUFSIZE*2]; /* huge buffer when doing upload CRLF replacing */
+};
+
+
+/*
+ * This 'DynamicStatic' struct defines dynamic states that actually change
+ * values in the 'UserDefined' area, which MUST be taken into consideration
+ * if the UserDefined struct is cloned or similar. You can probably just
+ * copy these, but each one indicate a special action on other data.
+ */
+
+struct DynamicStatic {
+  char *url;        /* work URL, copied from UserDefined */
+  bool url_alloc;   /* URL string is malloc()'ed */
+  char *proxy;      /* work proxy, copied from UserDefined */
+  bool proxy_alloc; /* http proxy string is malloc()'ed */
+  char *referer;    /* referer string */
+  bool referer_alloc; /* referer sting is malloc()ed */
+};
+
+/*
+ * This 'UserDefined' struct must only contain data that is set once to go
+ * for many (perhaps) independent connections. Values that are generated or
+ * calculated internally for the "session handle" MUST be defined within the
+ * 'struct urlstate' instead. The only exceptions MUST note the changes in
+ * the 'DynamicStatic' struct.
+ */
+
+struct UserDefined {
+  FILE *err;    /* the stderr writes goes here */
+  char *errorbuffer; /* store failure messages in here */
+  char *proxyuserpwd;  /* Proxy <user:password>, if used */
+  long proxyport; /* If non-zero, use this port number by default. If the
+                     proxy string features a ":[port]" that one will override
+                     this. */  
+  void *out;         /* the fetched file goes here */
+  void *in;          /* the uploaded file is read from here */
+  void *writeheader; /* write the header to this is non-NULL */
+  char *set_url;     /* what original URL to work on */
+  char *set_proxy;   /* proxy to use */
+  long use_port;     /* which port to use (when not using default) */
+  char *userpwd;     /* <user:password>, if used */
+  char *set_range;   /* range, if used. See README for detailed specification
+                        on this syntax. */
+  long followlocation; /* as in HTTP Location: */
+  long maxredirs;    /* maximum no. of http(s) redirects to follow */
+  char *set_referer; /* custom string */
+  bool free_referer; /* set TRUE if 'referer' points to a string we
+                        allocated */
+  char *useragent;   /* User-Agent string */
+  char *postfields;  /* if POST, set the fields' values here */
+  size_t postfieldsize; /* if POST, this might have a size to use instead of
+                           strlen(), and then the data *may* be binary (contain
+                           zero bytes) */
+  char *ftpport;     /* port to send with the FTP PORT command */
+  char *device;      /* network interface to use */
+  curl_write_callback fwrite;        /* function that stores the output */
+  curl_write_callback fwrite_header; /* function that stores headers */
+  curl_read_callback fread;          /* function that reads the input */
+  curl_progress_callback fprogress;  /* function for progress information */
+  void *progress_client; /* pointer to pass to the progress callback */
+  curl_passwd_callback fpasswd;      /* call for password */
+  void *passwd_client;               /* pass to the passwd callback */
+  long timeout;         /* in seconds, 0 means no timeout */
+  long connecttimeout;  /* in seconds, 0 means no timeout */
+  long infilesize;      /* size of file to upload, -1 means unknown */
+  long low_speed_limit; /* bytes/second */
+  long low_speed_time;  /* number of seconds */
+  int set_resume_from;  /* continue [ftp] transfer from here */
+  char *cookie;         /* HTTP cookie string to send */
+  struct curl_slist *headers; /* linked list of extra headers */
+  struct HttpPost *httppost;  /* linked list of POST data */
+  char *cert;           /* PEM-formatted certificate */
+  char *cert_passwd;    /* plain text certificate password */
+  char *cookiejar;      /* dump all cookies to this file */
+  bool crlf;            /* convert crlf on ftp upload(?) */
+  struct curl_slist *quote;     /* before the transfer */
+  struct curl_slist *postquote; /* after the transfer */
+  struct curl_slist *telnet_options; /* linked list of telnet options */
+  TimeCond timecondition; /* kind of time/date comparison */
+  time_t timevalue;       /* what time to compare with */
+  curl_closepolicy closepolicy; /* connection cache close concept */
+  Curl_HttpReq httpreq;   /* what kind of HTTP request (if any) is this */
+  char *customrequest;    /* HTTP/FTP request to use */
+  char *auth_host; /* if set, this is the allocated string to the host name
+                    * to which to send the authorization data to, and no other
+                    * host (which location-following otherwise could lead to)
+                    */
+  char *krb4_level; /* what security level */
+  struct ssl_config_data ssl;  /* user defined SSL stuff */
+
+/* Here follows boolean settings that define how to behave during
+   this session. They are STATIC, set by libcurl users or at least initially
+   and they don't change during operations. */
 
   bool get_filetime;
   bool tunnel_thru_httpproxy;
@@ -397,205 +547,39 @@ struct Configbits {
   bool http_include_header;
   bool http_set_referer;
   bool http_auto_referer; /* set "correct" referer when following location: */
-  bool httpproxy;
-  bool mute;
   bool no_body;
   bool set_port;
-  bool set_range;
   bool upload;
   bool use_netrc;
   bool verbose;
-  bool this_is_a_follow; /* this is a followed Location: request */
-  bool krb4; /* kerberos4 connection requested */
-  bool proxystringalloc; /* the http proxy string is malloc()'ed */
-  bool urlstringalloc;   /* the URL string is malloc()'ed */
-  bool reuse_forbid;     /* if this is forbidden to be reused, close 
-                            after use */
-  bool reuse_fresh;      /* do not re-use an existing connection for this
-                            transfer */
+  bool krb4;             /* kerberos4 connection requested */
+  bool reuse_forbid;     /* forbidden to be reused, close after use */
+  bool reuse_fresh;      /* do not re-use an existing connection  */
+  bool expect100header;  /* TRUE if we added Expect: 100-continue */
 };
 
 /*
- * As of April 11, 2000 we're now trying to split up the urldata struct in
- * three different parts:
+ * In August 2001, this struct was redesigned and is since stricter than
+ * before. The 'connectdata' struct MUST have all the connection oriented
+ * stuff as we may now have several simultaneous connections and connection
+ * structs in memory.
  *
- * (Global)
- * 1 - No matter how many hosts and requests that are being performed, this
- *     goes for all of them.
- *
- * (Session)
- * 2 - Host and protocol-specific. No matter if we do several transfers to and
- *     from this host, these variables stay the same.
- *
- * (Request)
- * 3 - Request-specific. Variables that are of interest for this particular
- *     transfer being made right now. THIS IS WRONG STRUCT FOR THOSE.
- *
- * In Febrary 2001, this is being done stricter. The 'connectdata' struct
- * MUST have all the connection oriented stuff as we may now have several
- * simultaneous connections and connection structs in memory.
- *
- * From now on, the 'UrlData' must only contain data that is set once to go
- * for many (perhaps) independent connections. Values that are generated or
- * calculated internally MUST NOT be a part of this struct.
- */
+ * From now on, the 'SessionHandle' must only contain data that is set once to
+ * go for many (perhaps) independent connections. Values that are generated or
+ * calculated internally for the "session handle" must be defined within the
+ * 'struct urlstate' instead.  */
 
-struct UrlData {
-  /*************** Global - specific items  ************/
-  FILE *err;    /* the stderr writes goes here */
-  char *errorbuffer; /* store failure messages in here */
-
-  /*************** Session - specific items ************/
-  char *proxy; /* if proxy, set it here */
-  char *proxyuserpwd;  /* Proxy <user:password>, if used */
-  long proxyport; /* If non-zero, use this port number by default. If the
-                     proxy string features a ":[port]" that one will override
-                     this. */
-
-  
-  long header_size;  /* size of read header(s) in bytes */
-  long request_size; /* the amount of bytes sent in the request(s) */
-
-  void *out;         /* the fetched file goes here */
-  void *in;          /* the uploaded file is read from here */
-  void *writeheader; /* write the header to this is non-NULL */
-
-  char *url;   /* what to get */
-  char *freethis; /* if non-NULL, an allocated string for the URL */
-  long use_port;  /* which port to use (when not using default) */
-  struct Configbits bits; /* new-style (v7) flag data */
-  struct ssl_config_data ssl; /* this is for ssl-stuff */
-
-  char *userpwd;  /* <user:password>, if used */
-  char *set_range; /* range, if used. See README for detailed specification on
-                      this syntax. */
-
-  /* stuff related to HTTP */
-
-  long followlocation;
-  long maxredirs; /* maximum no. of http(s) redirects to follow */
-  char *referer;
-  bool free_referer; /* set TRUE if 'referer' points to a string we
-                        allocated */
-  char *useragent;   /* User-Agent string */
-  char *postfields; /* if POST, set the fields' values here */
-  long postfieldsize; /* if POST, this might have a size to use instead of
-                         strlen(), and then the data *may* be binary (contain
-                         zero bytes) */
-
-  /* stuff related to FTP */
-  char *ftpport; /* port to send with the PORT command */
-
-  /* general things */
-  char *device;  /* Interface to use */
-
-  /* function that stores the output:*/
-  curl_write_callback fwrite;
-
-  /* optional function that stores the header output:*/
-  curl_write_callback fwrite_header;
-
-  /* function that reads the input:*/
-  curl_read_callback fread;
-
-  /* function that wants progress information */
-  curl_progress_callback fprogress;
-  void *progress_client; /* pointer to pass to the progress callback */
-
-  /* function to call instead of the internal for password */
-  curl_passwd_callback fpasswd;
-  void *passwd_client; /* pointer to pass to the passwd callback */
-
-  long timeout;        /* in seconds, 0 means no timeout */
-  long connecttimeout; /* in seconds, 0 means no timeout */
-  long infilesize;     /* size of file to upload, -1 means unknown */
-
-  char buffer[BUFSIZE+1]; /* buffer with size BUFSIZE */
-
-  double current_speed;  /* the ProgressShow() funcion sets this */
-
-  long low_speed_limit; /* bytes/second */
-  long low_speed_time;  /* number of seconds */
-
-  int set_resume_from;    /* continue [ftp] transfer from here */
-
-  char *cookie;       /* HTTP cookie string to send */
-
-  struct curl_slist *headers; /* linked list of extra headers */
-  struct HttpPost *httppost;  /* linked list of POST data */
-
-  char *cert; /* PEM-formatted certificate */
-  char *cert_passwd; /* plain text certificate password */
-
-  struct CookieInfo *cookies;
-
-  long crlf;
-  struct curl_slist *quote;     /* before the transfer */
-  struct curl_slist *postquote; /* after the transfer */
-
-  /* Telnet negotiation options */
-  struct curl_slist *telnet_options; /* linked list of telnet options */
-
-  TimeCond timecondition; /* kind of comparison */
-  time_t timevalue;       /* what time to compare with */
-
-  Curl_HttpReq httpreq; /* what kind of HTTP request (if any) is this */
-
-  char *customrequest; /* http/ftp request to use */
-
-  char *headerbuff; /* allocated buffer to store headers in */
-  int headersize;   /* size of the allocation */
-
-  struct Progress progress; /* for all the progress meter data */
-
-#define MAX_CURL_USER_LENGTH 128
-#define MAX_CURL_PASSWORD_LENGTH 128
-
-  char *auth_host; /* if set, this is the allocated string to the host name
-                    * to which to send the authorization data to, and no other
-                    * host (which location-following otherwise could lead to)
-                    */
-
-  /* buffers to store authentication data in */
-  char user[MAX_CURL_USER_LENGTH];
-  char passwd[MAX_CURL_PASSWORD_LENGTH];
-  char proxyuser[MAX_CURL_USER_LENGTH];
-  char proxypasswd[MAX_CURL_PASSWORD_LENGTH];
-
-  char *krb4_level; /* what security level */
-  struct timeval keeps_speed; /* this should be request-specific */
-
-  /* 'connects' will be an allocated array with pointers. If the pointer is
-     set, it holds an allocated connection. */
-  struct connectdata **connects;
-  size_t numconnects; /* size of the 'connects' array */
-  curl_closepolicy closepolicy;
+struct SessionHandle {
+  struct UserDefined set;      /* values set by the libcurl user */
+  struct DynamicStatic change; /* possibly modified userdefined data */
 
+  struct CookieInfo *cookies;  /* the cookies, read from files and servers */
+  struct Progress progress;    /* for all the progress meter data */
+  struct UrlState state;       /* struct for fields used for state info and
+                                  other dynamic purposes */
+  struct PureInfo info;        /* stats, reports and info data */
 };
 
 #define LIBCURL_NAME "libcurl"
-#define LIBCURL_ID LIBCURL_NAME " " LIBCURL_VERSION " " SSL_ID
-
-CURLcode Curl_getinfo(CURL *curl, CURLINFO info, ...);
-
-/*
- * Here follows function prototypes from what we used to plan to call
- * the "low level" interface. It is no longer prioritized and it is not likely
- * to ever be supported to external users.
- *
- * I removed all the comments to them as well, as they were no longer accurate
- * and they're not meant for "public use" anymore.
- */
-
-CURLcode Curl_open(CURL **curl, char *url);
-CURLcode Curl_setopt(CURL *handle, CURLoption option, ...);
-CURLcode Curl_close(CURL *curl); /* the opposite of curl_open() */
-CURLcode Curl_connect(struct UrlData *,
-                      struct connectdata **,
-                      bool allow_port);
-CURLcode Curl_do(struct connectdata *);
-CURLcode Curl_done(struct connectdata *);
-CURLcode Curl_disconnect(struct connectdata *);
-
 
 #endif
