@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 2001, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * In order to be useful for every potential user, curl and libcurl are
  * dual-licensed under the MPL and the MIT/X-derivate licenses.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: main.c,v 1.77 2001/05/04 07:47:11 bagder Exp $
+ * $Id: main.c,v 1.95 2001/09/11 22:36:03 bagder Exp $
  *****************************************************************************/
 
 /* This is now designed to have its own local setup.h */
@@ -60,6 +60,14 @@
 #include <fcntl.h>
 #endif
 
+#ifdef HAVE_UTIME_H
+ #include <utime.h>
+#else
+ #ifdef HAVE_SYS_UTIME_H
+ #include <sys/utime.h>
+ #endif
+#endif
+
 /* The last #include file should be: */
 #ifdef MALLOCDEBUG
 /* this is low-level hard-hacking memory leak tracking shit */
@@ -88,7 +96,6 @@ typedef enum {
 /* Just a set of bits */
 #define CONF_DEFAULT  0
 
-#define CONF_USEREMOTETIME (1<<0) /* set the remote time on the local file */
 #define CONF_AUTO_REFERER (1<<4) /* the automatic referer-system please! */
 #define CONF_VERBOSE  (1<<5) /* talk a lot */
 #define CONF_HEADER   (1<<8) /* throw the header out too */
@@ -124,53 +131,116 @@ char *strdup(char *str)
 }
 #endif 
 
-extern void hugehelp(void);
-
-/***********************************************************************
- * Start with some silly functions to make win32-systems survive
- ***********************************************************************/
-#if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
-static void win32_cleanup(void)
-{
-  WSACleanup();
-}
-
-static CURLcode win32_init(void)
-{
-  WORD wVersionRequested;  
-  WSADATA wsaData; 
-  int err; 
-  wVersionRequested = MAKEWORD(1, 1); 
-    
-  err = WSAStartup(wVersionRequested, &wsaData); 
-    
-  if (err != 0) 
-    /* Tell the user that we couldn't find a useable */ 
-    /* winsock.dll.     */ 
-    return CURLE_FAILED_INIT; 
-    
-  /* Confirm that the Windows Sockets DLL supports 1.1.*/ 
-  /* Note that if the DLL supports versions greater */ 
-  /* than 1.1 in addition to 1.1, it will still return */ 
-  /* 1.1 in wVersion since that is the version we */ 
-  /* requested. */ 
-    
-  if ( LOBYTE( wsaData.wVersion ) != 1 || 
-       HIBYTE( wsaData.wVersion ) != 1 ) { 
-    /* Tell the user that we couldn't find a useable */ 
-
-    /* winsock.dll. */ 
-    WSACleanup(); 
-    return CURLE_FAILED_INIT; 
-  }
-  return CURLE_OK;
-}
-/* The Windows Sockets DLL is acceptable. Proceed. */ 
-#else
-static CURLcode win32_init(void) { return CURLE_OK; }
-#define win32_cleanup()
+#ifdef	VMS
+int	vms_show = 0;
+#define	FAC_CURL	0xC01
+#define	FAC_SYSTEM	0
+#define	MSG_NORMAL	0
+#define	VMS_STS(c,f,e,s) (((c&0xF)<<28)|((f&0xFFF)<<16)|((e&0x1FFF)<3)|(s&7))
+#define	VMSSTS_HIDE	VMS_STS(1,0,0,0)
+#define	SEV_WARNING	0
+#define	SEV_SUCCESS	1
+#define	SEV_ERROR	2
+#define	SEV_INFO	3	/* success, with an extra hint */
+#define	SEV_FATAL	4
+globalvalue int  CURL_UNSUPPROTO;		/* these are from curlmsg.msg file..... */
+globalvalue int  CURL_FAILINIT;
+globalvalue int  CURL_BADURLSYN;
+globalvalue int  CURL_BADURLUSER;
+globalvalue int  CURL_BADPROXY;
+globalvalue int  CURL_BADHOST;
+globalvalue int  CURL_FAILHOST;
+globalvalue int  CURL_FTPUNKREPLY;
+globalvalue int  CURL_FTPNOACC;
+globalvalue int  CURL_FTPUSRPW;
+globalvalue int  CURL_FTPBADPASS;
+globalvalue int  CURL_FTPBADUSER;
+globalvalue int  CURL_FTPBADPASV;
+globalvalue int  CURL_FTPBAD227;
+globalvalue int  CURL_FTPBADHOST227;
+globalvalue int  CURL_FTPNORECONN;
+globalvalue int  CURL_FTPNOBIN;
+globalvalue int  CURL_PARTIALFILE;
+globalvalue int  CURL_FTPNORETR;
+globalvalue int  CURL_FTPWRITERR;
+globalvalue int  CURL_FTPNOQUOTE;
+globalvalue int  CURL_HTTPPNF;
+globalvalue int  CURL_WRITERR;
+globalvalue int  CURL_BADUSER;
+globalvalue int  CURL_FTPNOSTOR;
+globalvalue int  CURL_READERR;
+globalvalue int  CURL_OUTOFMEM;
+globalvalue int  CURL_TIMEOUT;
+globalvalue int  CURL_FTPNOASCII;
+globalvalue int  CURL_FTPNOPORT;
+globalvalue int  CURL_FTPNOREST;
+globalvalue int  CURL_FTPNOSIZE;
+globalvalue int  CURL_HTTPRNGERR;
+globalvalue int  CURL_HTTPPOSTERR;
+globalvalue int  CURL_SSLNOCONN;
+globalvalue int  CURL_FTPBADRESUME;
+globalvalue int  CURL_FILENOACC;
+globalvalue int  CURL_LDAPNOBIND;
+globalvalue int  CURL_LDAPNOSRCH;
+globalvalue int  CURL_LDAPNOLIB;
+globalvalue int  CURL_LDAPNOFUNC;
+globalvalue int  CURL_ABORTCB;
+globalvalue int  CURL_BADPARAM;
+globalvalue int  CURL_BADORDER;
+globalvalue int  CURL_BADPWD;
+globalvalue int  CURL_MNYREDIR;
+long	vms_cond[] = {
+	VMS_STS(1,FAC_SYSTEM,MSG_NORMAL,SEV_SUCCESS),
+	CURL_UNSUPPROTO,		/* these are from curlmsg.msg file..... */
+	CURL_FAILINIT,
+	CURL_BADURLSYN,
+	CURL_BADURLUSER,
+	CURL_BADPROXY,
+	CURL_BADHOST,
+	CURL_FAILHOST,
+	CURL_FTPUNKREPLY,
+	CURL_FTPNOACC,
+	CURL_FTPUSRPW,
+	CURL_FTPBADPASS,
+	CURL_FTPBADUSER,
+	CURL_FTPBADPASV,
+	CURL_FTPBAD227,
+	CURL_FTPBADHOST227,
+	CURL_FTPNORECONN,
+	CURL_FTPNOBIN,
+	CURL_PARTIALFILE,
+	CURL_FTPNORETR,
+	CURL_FTPWRITERR,
+	CURL_FTPNOQUOTE,
+	CURL_HTTPPNF,
+	CURL_WRITERR,
+	CURL_BADUSER,
+	CURL_FTPNOSTOR,
+	CURL_READERR,
+	CURL_OUTOFMEM,
+	CURL_TIMEOUT,
+	CURL_FTPNOASCII,
+	CURL_FTPNOPORT,
+	CURL_FTPNOREST,
+	CURL_FTPNOSIZE,
+	CURL_HTTPRNGERR,
+	CURL_HTTPPOSTERR,
+	CURL_SSLNOCONN,
+	CURL_FTPBADRESUME,
+	CURL_FILENOACC,
+	CURL_LDAPNOBIND,
+	CURL_LDAPNOSRCH,
+	CURL_LDAPNOLIB,
+	CURL_LDAPNOFUNC,
+	CURL_ABORTCB,
+	CURL_BADPARAM,
+	CURL_BADORDER,
+	CURL_BADPWD,
+	CURL_MNYREDIR
+};
 #endif
 
+extern void hugehelp(void);
 
 /*
  * This is the main global constructor for the app. Call this before
@@ -179,7 +249,7 @@ static CURLcode win32_init(void) { return CURLE_OK; }
  */
 CURLcode main_init(void)
 {
-  return win32_init();
+  return curl_global_init(CURL_GLOBAL_DEFAULT);
 }
 
 /*
@@ -188,7 +258,7 @@ CURLcode main_init(void)
  */
 void main_free(void)
 {
-  win32_cleanup();
+  curl_global_cleanup();
 }
 
 int SetHTTPrequest(HttpReq req, HttpReq *store)
@@ -202,7 +272,7 @@ int SetHTTPrequest(HttpReq req, HttpReq *store)
   return 1;
 }
 
-static void helpf(char *fmt, ...)
+static void helpf(const char *fmt, ...)
 {
   va_list ap;
   if(fmt) {
@@ -238,7 +308,8 @@ static void help(void)
        " -b/--cookie <name=string/file> Cookie string or file to read cookies from (H)\n"
        " -B/--use-ascii     Use ASCII/text transfer\n",
          curl_version());
-  puts(" -C/--continue-at <offset> Specify absolute resume offset\n"
+  puts(" -c/--cookie-jar <file> Write all cookies to this file after operation (H)\n"
+       " -C/--continue-at <offset> Specify absolute resume offset\n"
        " -d/--data <data>   HTTP POST data (H)\n"
        "    --data-ascii <data>   HTTP POST ASCII data (H)\n"
        "    --data-binary <data>  HTTP POST binary data (H)\n"
@@ -246,14 +317,16 @@ static void help(void)
        "    --egd-file <file> EGD socket path for random data (SSL)\n"
        " -e/--referer       Referer page (H)");
   puts(" -E/--cert <cert[:passwd]> Specifies your certificate file and password (HTTPS)\n"
-       "    --cacert <file> CA certifciate to verify peer against (HTTPS)\n"
+       "    --cacert <file> CA certifciate to verify peer against (SSL)\n"
+       "    --ciphers <list> What SSL ciphers to use (SSL)\n"
        "    --connect-timeout <seconds> Maximum time allowed for connection\n"
        " -f/--fail          Fail silently (no output at all) on errors (H)\n"
        " -F/--form <name=content> Specify HTTP POST data (H)\n"
        " -g/--globoff       Disable URL sequences and ranges using {} and []\n"
-       " -h/--help          This help text\n"
-       " -H/--header <line> Custom header to pass to server. (H)");
-  puts(" -i/--include       Include the HTTP-header in the output (H)\n"
+       " -G/--get           Send the -d data with a HTTP GET (H)");
+  puts(" -h/--help          This help text\n"
+       " -H/--header <line> Custom header to pass to server. (H)\n"
+       " -i/--include       Include the HTTP-header in the output (H)\n"
        " -I/--head          Fetch document info only (HTTP HEAD/FTP SIZE)\n"
        "    --interface <interface> Specify the interface to be used\n"
        "    --krb4 <level>  Enable krb4 with specified security level (F)\n"
@@ -271,6 +344,7 @@ static void help(void)
        " -q                 When used as the first parameter disables .curlrc\n"
        " -Q/--quote <cmd>   Send QUOTE command to FTP before file transfer (F)");
   puts(" -r/--range <range> Retrieve a byte range from a HTTP/1.1 or FTP server\n"
+       " -R/--remote-time   Set the remote file's time on the local output\n"
        " -s/--silent        Silent mode. Don't output anything\n"
        " -S/--show-error    Show error. With -s, make curl show errors when they occur\n"
        "    --stderr <file> Where to redirect stderr. - means stdout.\n"
@@ -296,17 +370,21 @@ static void help(void)
 }
 
 struct LongShort {
-  char *letter;
-  char *lname;
+  const char *letter;
+  const char *lname;
   bool extraparam;
 };
 
 struct Configurable {
+  bool remote_time;
   char *random_file;
   char *egd_file;
   char *useragent;
-  char *cookie;
+  char *cookie;     /* single line with specified cookies */
+  char *cookiejar;  /* write to this file */
+  char *cookiefile; /* read from this file */
   bool use_resume;
+  bool resume_from_current;
   int resume_from;
   char *postfields;
   long postfieldsize;
@@ -336,16 +414,17 @@ struct Configurable {
   struct getout *url_get;  /* point to the node to fill in URL */
   struct getout *url_out;  /* point to the node to fill in outfile */
 
+  char *cipher_list;
   char *cert;
   char *cacert;
   char *cert_passwd;
   bool crlf;
-  char *cookiefile;
   char *customrequest;
   char *krb4level;
   bool progressmode;
   bool nobuffer;
   bool globoff;
+  bool use_httpget;
 
   char *writeout; /* %-styled format string to output */
 
@@ -505,9 +584,9 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
   char letter;
   char subletter=0; /* subletters can only occur on long options */
 
-  char *parse=NULL;
+  const char *parse=NULL;
   int res;
-  int j;
+  unsigned int j;
   time_t now;
   int hit=-1;
   bool longopt=FALSE;
@@ -527,15 +606,16 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     {"5a", "random-file", TRUE},
     {"5b", "egd-file",   TRUE},
     {"5c", "connect-timeout", TRUE},
+    {"5d", "ciphers",    TRUE},
 
     {"2", "sslv2",       FALSE},
     {"3", "sslv3",       FALSE},
     {"a", "append",      FALSE},
     {"A", "user-agent",  TRUE},
     {"b", "cookie",      TRUE},
-    {"B", "ftp-ascii",   FALSE}, /* this long format is OBSOLETEE now! */
+    {"B", "ftp-ascii",   FALSE}, /* this long format is OBSOLETE now! */
     {"B", "use-ascii",   FALSE},
-    {"c", "continue",    FALSE},
+    {"c", "cookie-jar",  TRUE},
     {"C", "continue-at", TRUE},
     {"d", "data",        TRUE},
     {"da", "data-ascii", TRUE},
@@ -547,6 +627,7 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     {"f", "fail",        FALSE},
     {"F", "form",        TRUE},
     {"g", "globoff",     FALSE},
+    {"G", "get",         FALSE},
     {"h", "help",        FALSE},
     {"H", "header",      TRUE},
     {"i", "include",     FALSE},
@@ -565,6 +646,7 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     {"q", "disable",     FALSE},
     {"Q", "quote",       TRUE},
     {"r", "range",       TRUE},
+    {"R", "remote-time", FALSE},
     {"s", "silent",      FALSE},
     {"S", "show-error",  FALSE},
     {"t", "telnet-options", TRUE},
@@ -647,7 +729,7 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       return PARAM_OPTION_UNKNOWN;
     }    
     if(!longopt && aliases[hit].extraparam && parse[1]) {
-      nextarg=&parse[1]; /* this is the actual extra parameter */
+      nextarg=(char *)&parse[1]; /* this is the actual extra parameter */
       singleopt=TRUE;   /* don't loop anymore after this */
     }
     else if((!nextarg || !*nextarg) && aliases[hit].extraparam) {
@@ -686,6 +768,9 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       case 'c': /* connect-timeout */
         config->connecttimeout=atoi(nextarg);
         break;
+      case 'd': /* ciphers */
+        GetStr(&config->cipher_list, nextarg);
+        break;
       default: /* the URL! */
         {
           struct getout *url;
@@ -749,14 +834,19 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       config->conf ^= CONF_GETTEXT;
       break;
     case 'c':
-      /* This makes us continue an ftp transfer */
-      config->use_resume^=TRUE;
-      fprintf(stderr, "-c is a deprecated switch, use '-C -' instead!\n");
+      /* get the file name to dump all cookies in */
+      GetStr(&config->cookiejar, nextarg);
       break;
     case 'C':
       /* This makes us continue an ftp transfer at given position */
-      if(!strequal(nextarg, "-"))
+      if(!strequal(nextarg, "-")) {
         config->resume_from= atoi(nextarg);
+        config->resume_from_current = FALSE;
+      }
+      else {
+        config->resume_from_current = TRUE;
+        config->resume_from = 0;
+      }
       config->use_resume=TRUE;
       break;
     case 'd':
@@ -799,8 +889,8 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
           config->postfields=postdata;
       }
 
-      if(SetHTTPrequest(HTTPREQ_SIMPLEPOST, &config->httpreq))
-        return PARAM_BAD_USE;
+/*      if(SetHTTPrequest(HTTPREQ_SIMPLEPOST, &config->httpreq))
+        return PARAM_BAD_USE;*/
       break;
     case 'D':
       /* dump-header to given file name */
@@ -825,7 +915,23 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       }
       else {
 	char *ptr = strchr(nextarg, ':');
-	if(ptr) {
+        /* Since we live in a world of weirdness and confusion, the win32
+           dudes can use : when using drive letters and thus
+           c:\file:password needs to work. In order not to break
+           compatibility, we still use : as separator, but we try to detect
+           when it is used for a file name! On windows. */
+#ifdef WIN32
+        if(ptr &&
+           (ptr == &nextarg[1]) &&
+           (nextarg[2] == '\\') &&
+           (isalpha((int)nextarg[0])) )
+          /* colon in the second column, followed by a backslash, and the
+             first character is an alphabetic letter:
+
+             this is a drive letter colon */
+          ptr = strchr(&nextarg[3], ':'); /* find the next one instead */
+#endif
+        if(ptr) {
 	  /* we have a password too */
 	  *ptr=0;
 	  ptr++;
@@ -853,6 +959,10 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       config->globoff ^= TRUE;
       break;
 
+    case 'G': /* HTTP GET */
+      config->use_httpget = TRUE;
+      break;
+
     case 'h': /* h for help */
       help();
       return PARAM_HELP_REQUESTED;
@@ -961,6 +1071,10 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       /* byte range requested */
       GetStr(&config->range, nextarg);
       break;
+    case 'R':
+      /* use remote file's time */
+      config->remote_time ^= TRUE;
+      break;
     case 's':
       /* don't show progress meter, don't show errors : */
       config->conf |= (CONF_MUTE|CONF_NOPROGRESS);
@@ -1056,7 +1170,7 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       }
       now=time(NULL);
       config->condtime=curl_getdate(nextarg, &now);
-      if(-1 == config->condtime) {
+      if(-1 == (int)config->condtime) {
         /* now let's see if it is a file name to get the time from instead! */
         struct stat statbuf;
         if(-1 == stat(nextarg, &statbuf)) {
@@ -1100,7 +1214,6 @@ static int parseconfig(char *filename,
 #define CURLRC DOT_CHAR "curlrc"
 
     home = curl_getenv("HOME"); /* portable environment reader */
-
     if(!home)
       return 0;
     if(strlen(home)>(sizeof(filebuffer)-strlen(CURLRC))) {
@@ -1222,10 +1335,10 @@ static int parseconfig(char *filename,
       if(res != PARAM_OK) {
         /* the help request isn't really an error */
         if(!strcmp(filename, "-")) {
-          filename="<stdin>";
+          filename=(char *)"<stdin>";
         }
         if(PARAM_HELP_REQUESTED != res) {
-          char *reason;
+          const char *reason;
           switch(res) {
           default:
           case PARAM_GOT_EXTRA_PARAMETER:
@@ -1291,6 +1404,7 @@ struct ProgressData {
   size_t prev;
   size_t point;
   int width;
+  FILE *out; /* where to write everything to */
 };
 
 int myprogress (void *clientp,
@@ -1320,7 +1434,7 @@ int myprogress (void *clientp,
     int prevblock = bar->prev / 1024;
     int thisblock = bar->point / 1024;
     while ( thisblock > prevblock ) {
-      fprintf( stderr, "#" );
+      fprintf( bar->out, "#" );
       prevblock++;
     }
   }
@@ -1336,14 +1450,16 @@ int myprogress (void *clientp,
     line[i] = '\0';
     sprintf( format, "%%-%ds %%5.1f%%%%", barwidth );
     sprintf( outline, format, line, percent );
-    fprintf( stderr, "\r%s", outline );
+    fprintf( bar->out, "\r%s", outline );
   }
   bar->prev = bar->point;
 
   return 0;
 }
 
-void progressbarinit(struct ProgressData *bar)
+static
+void progressbarinit(struct ProgressData *bar,
+                     struct Configurable *config)
 {
 #ifdef __EMX__
   /* 20000318 mgs */
@@ -1377,6 +1493,7 @@ void progressbarinit(struct ProgressData *bar)
   bar->width = scr_size[0] - 1;
 #endif
 
+  bar->out = config->errors;
 }
 
 void free_config_fields(struct Configurable *config)
@@ -1449,6 +1566,8 @@ operate(struct Configurable *config, int argc, char *argv[])
 
   bool allocuseragent=FALSE;
 
+  char *httpgetfields=NULL;
+
   CURL *curl;
   int res;
   int i;
@@ -1458,10 +1577,11 @@ operate(struct Configurable *config, int argc, char *argv[])
   curl_memdebug("memdump");
 #endif
 
-  main_init(); /* inits winsock crap for windows */
+  main_init(); /* inits */
 
   config->showerror=TRUE;
   config->conf=CONF_DEFAULT;
+  config->use_httpget=FALSE;
 
   if(argc>1 &&
      (!strnequal("--", argv[1], 2) && (argv[1][0] == '-')) &&
@@ -1530,7 +1650,7 @@ operate(struct Configurable *config, int argc, char *argv[])
     else {
       bool used;
       /* just add the URL please */
-      res = getparameter("--url", argv[i], &used, config);
+      res = getparameter((char *)"--url", argv[i], &used, config);
       if(res)
         return res;
     }
@@ -1549,6 +1669,24 @@ operate(struct Configurable *config, int argc, char *argv[])
   else
     allocuseragent = TRUE;
 
+  if (config->postfields) {
+    if (config->use_httpget) {
+      /* Use the postfields data for a http get */
+      httpgetfields = strdup(config->postfields);
+      free(config->postfields);
+      config->postfields = NULL;
+      if(SetHTTPrequest((config->conf&CONF_NOBODY?HTTPREQ_HEAD:HTTPREQ_GET),
+                        &config->httpreq)) {
+        free(httpgetfields);
+        return PARAM_BAD_USE;
+      }
+    }
+    else {
+      if(SetHTTPrequest(HTTPREQ_SIMPLEPOST, &config->httpreq))
+        return PARAM_BAD_USE;
+    }
+  }
+
   /*
    * Get a curl handle to use for all forthcoming curl transfers.  Cleanup
    * when all transfers are done. This is supported with libcurl 7.7 and
@@ -1633,12 +1771,13 @@ operate(struct Configurable *config, int argc, char *argv[])
           free(storefile);
         }
       
-        if((0 == config->resume_from) && config->use_resume) {
+        if(config->resume_from_current) {
           /* we're told to continue where we are now, then we get the size of
              the file as it is now and open it for append instead */
 
           struct stat fileinfo;
 
+/*VMS?? -- Danger, the filesize is only valid for stream files */
           if(0 == stat(outfile, &fileinfo)) {
             /* set offset to current file size: */
             config->resume_from = fileinfo.st_size;
@@ -1691,6 +1830,15 @@ operate(struct Configurable *config, int argc, char *argv[])
           
           url = urlbuffer; /* use our new URL instead! */
         }
+/*VMS??-- Reading binary from files can be a problem... */
+/*VMS??   Only FIXED, VAR etc WITHOUT implied CC will work */
+/*VMS??   Others need a \n appended to a line */
+/*VMS??-- Stat gives a size but this is UNRELIABLE in VMS */
+/*VMS??   As a f.e. a fixed file with implied CC needs to have a byte added */
+/*VMS??   for every record processed, this can by derived from Filesize & recordsize */
+/*VMS??   for VARiable record files the records need to be counted! */
+/*VMS??   for every record add 1 for linefeed and subtract 2 for the record header */
+/*VMS??   for VARIABLE header files only the bare record data needs to be considered with one appended if implied CC */
 
         infd=(FILE *) fopen(config->infile, "rb");
         if (!infd || stat(config->infile, &fileinfo)) {
@@ -1701,8 +1849,7 @@ operate(struct Configurable *config, int argc, char *argv[])
       
       }
       if((config->conf&CONF_UPLOAD) &&
-         config->use_resume &&
-         (0==config->resume_from)) {
+         config->resume_from_current) {
         config->resume_from = -1; /* -1 will then force get-it-yourself */
       }
       if(config->headerfile) {
@@ -1730,6 +1877,44 @@ operate(struct Configurable *config, int argc, char *argv[])
         if (separator)
           printf("%s%s\n", CURLseparator, url);
       }
+      if (httpgetfields) {
+        /* Find out whether the url contains a file name */
+        char *pc =strstr(url, "://");
+        char separator='?';
+        if(pc)
+          pc+=3;
+        else
+          pc=url;
+
+        pc = strrchr(pc, '/'); /* check for a slash */
+
+        if(pc) {
+          /* there is a slash present in the URL */
+
+          if(strchr(pc, '?'))
+          /* Ouch, there's already a question mark in the URL string, we
+             then appead the data with an amperand separator instead! */
+            separator='&';
+        }
+        /*
+         * Then append ? followed by the get fields to the url.
+         */
+        urlbuffer=(char *)malloc(strlen(url) + strlen(httpgetfields) + 2);
+        if(!urlbuffer) {
+          helpf("out of memory\n");
+          return CURLE_OUT_OF_MEMORY;
+        }
+        if (pc)
+          sprintf(urlbuffer, "%s%c%s", url, separator, httpgetfields);
+        else
+          /* Append  / before the ? to create a well-formed url
+             if the url contains a hostname only
+          */
+          sprintf(urlbuffer, "%s/?%s", url, httpgetfields);
+ 
+        free(url); /* free previous URL */
+        url = urlbuffer; /* use our new URL instead! */
+      }
 
       if(!config->errors)
         config->errors = stderr;
@@ -1792,18 +1977,19 @@ operate(struct Configurable *config, int argc, char *argv[])
       curl_easy_setopt(curl, CURLOPT_SSLCERTPASSWD, config->cert_passwd);
 
       if(config->cacert) {
-        /* available from libcurl 7.5: */
         curl_easy_setopt(curl, CURLOPT_CAINFO, config->cacert);
         curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, TRUE);
+        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 2);
       }
+      else
+        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 1);
       
-      if(config->conf&(CONF_NOBODY|CONF_USEREMOTETIME)) {
+      if((config->conf&CONF_NOBODY) ||
+         config->remote_time) {
         /* no body or use remote time */
-        /* new in 7.5 */
         curl_easy_setopt(curl, CURLOPT_FILETIME, TRUE);
       }
       
-      /* 7.5 news: */
       if (config->maxredirs) 
         curl_easy_setopt(curl, CURLOPT_MAXREDIRS, config->maxredirs); 
       else 
@@ -1815,6 +2001,8 @@ operate(struct Configurable *config, int argc, char *argv[])
       curl_easy_setopt(curl, CURLOPT_WRITEHEADER,
                        config->headerfile?&heads:NULL);
       curl_easy_setopt(curl, CURLOPT_COOKIEFILE, config->cookiefile);
+      /* cookie jar was added in 7.9 */
+      curl_easy_setopt(curl, CURLOPT_COOKIEJAR, config->cookiejar);
       curl_easy_setopt(curl, CURLOPT_SSLVERSION, config->ssl_version);
       curl_easy_setopt(curl, CURLOPT_TIMECONDITION, config->timecond);
       curl_easy_setopt(curl, CURLOPT_TIMEVALUE, config->condtime);
@@ -1830,7 +2018,7 @@ operate(struct Configurable *config, int argc, char *argv[])
          !(config->conf&(CONF_NOPROGRESS|CONF_MUTE))) {
         /* we want the alternative style, then we have to implement it
            ourselves! */
-        progressbarinit(&progressbar);
+        progressbarinit(&progressbar, config);
         curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, myprogress);
         curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, &progressbar);
       }
@@ -1842,21 +2030,26 @@ operate(struct Configurable *config, int argc, char *argv[])
       curl_easy_setopt(curl, CURLOPT_RANDOM_FILE, config->random_file);
       curl_easy_setopt(curl, CURLOPT_EGDSOCKET, config->egd_file);
       curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, config->connecttimeout);
+
+      /* debug */
+      if(config->cipher_list)
+        curl_easy_setopt(curl, CURLOPT_SSL_CIPHER_LIST, config->cipher_list);
       
       res = curl_easy_perform(curl);
         
       if(config->writeout) {
         ourWriteOut(curl, config->writeout);
       }
-        
+
+#ifdef	VMS
+      if (!config->showerror)  {
+        vms_show = VMSSTS_HIDE;
+      }
+#else
       if((res!=CURLE_OK) && config->showerror)
         fprintf(config->errors, "curl: (%d) %s\n", res, errorbuffer);
+#endif
 
-      if((config->errors != stderr) &&
-         (config->errors != stdout))
-        /* it wasn't directed to stdout or stderr so close the file! */
-        fclose(config->errors);
-    
       if(config->headerfile && !headerfilep && heads.stream)
         fclose(heads.stream);
 
@@ -1864,11 +2057,31 @@ operate(struct Configurable *config, int argc, char *argv[])
         free(urlbuffer);
       if (outfile && !strequal(outfile, "-") && outs.stream)
         fclose(outs.stream);
+
+#ifdef HAVE_UTIME
+      /* Important that we set the time _after_ the file has been 
+         closed, as is done above here */
+      if(config->remote_time && outs.filename) {
+        /* as libcurl if we got a time. Pretty please */
+        long filetime;
+        curl_easy_getinfo(curl, CURLINFO_FILETIME, &filetime);
+        if(filetime >= 0) {
+          struct utimbuf times;
+          times.actime = filetime;
+          times.modtime = filetime;
+          utime(outs.filename, &times); /* set the time we got */
+        }
+      }
+#endif
+
       if (config->infile)
         fclose(infd);
       if(headerfilep)
         fclose(headerfilep);
       
+      if (httpgetfields)
+        free(httpgetfields);
+
       if(url)
         free(url);
 
@@ -1901,7 +2114,12 @@ operate(struct Configurable *config, int argc, char *argv[])
   /* cleanup the curl handle! */
   curl_easy_cleanup(curl);
 
-  main_free(); /* cleanup the winsock stuff for windows */
+  if((config->errors != stderr) &&
+     (config->errors != stdout))
+    /* it wasn't directed to stdout or stderr so close the file! */
+    fclose(config->errors);
+
+  main_free(); /* cleanup */
 
   return res;
 }
@@ -1916,7 +2134,11 @@ int main(int argc, char *argv[])
   res = operate(&config, argc, argv);
   free_config_fields(&config);
 
+#ifdef	VMS
+  return (vms_cond[res]|vms_show);
+#else
   return res;
+#endif
 }
 
 static char *my_get_line(FILE *fp)
