@@ -7,38 +7,21 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <daniel@haxx.se>
- *
- * 	http://curl.haxx.se
- *
- * $Source: /cvsroot/curl/src/setup.h,v $
- * $Revision: 1.5 $
- * $Date: 2000/11/09 12:51:43 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- ****************************************************************************/
+ * $Id: setup.h,v 1.7 2001/09/17 14:10:38 bagder Exp $
+ *****************************************************************************/
 
 #include <stdio.h>
 
@@ -61,7 +44,7 @@
 #define OS "unknown"
 #endif
 
-#ifndef fileno /* sunos 4 have this as a macro! */
+#if !defined(fileno) && !defined(WIN32) /* sunos 4 have this as a macro! */
 int fileno( FILE *stream);
 #endif
 
